# The LearningOnline Network
# Pick a course
#
# $Id: lonpickcourse.pm,v 1.17 2004/06/29 14:56:32 raeburn Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::lonpickcourse;

use strict;
use Apache::Constants qw(:common);
use Apache::loncommon;
use Apache::loncoursedata;
use Apache::lonnet;
use Apache::lonlocal;

sub handler {
    my $r = shift;
    &Apache::loncommon::content_type($r,'text/html');
    $r->send_http_header;
    return OK if $r->header_only;

# ------------------------------------------------------------ Print the screen
    $r->print(<<ENDDOCUMENT);
<html>
<head>
<title>The LearningOnline Network with CAPA</title>
</head>
ENDDOCUMENT
# Get parameters from query string
    &Apache::loncommon::get_unprocessed_cgi
        ($ENV{'QUERY_STRING'},['domainfilter','descriptfilter',
			       'sincefilter','form','cnumelement',
			       'cdomelement','cnameelement']);
# domain filter and selection
    my $domainfilter=$ENV{'form.domainfilter'};
    $domainfilter=~s/\W//g;
    unless ($domainfilter) { $domainfilter=''; }
    my $domainselectform=&Apache::loncommon::select_dom_form($domainfilter,
                                                          'domainfilter',1);
# description filter
    my $descriptfilter=$ENV{'form.descriptfilter'};
    unless ($descriptfilter) { $descriptfilter=''; }
    my $listdescript=&HTML::Entities::encode($descriptfilter,'<>&"');
# last course activity filter and selection
    my $sincefilter=$ENV{'form.sincefilter'};
    $sincefilter=~s/[^\d-]//g;
    unless ($sincefilter) { $sincefilter=-1; }
    my $sincefilterform=&Apache::loncommon::select_form($sincefilter,
       'sincefilter',('-1'=>'',
                      '86400' => 'today',
                      '604800' => 'last week',
                      '2592000' => 'last month',
                      '7776000' => 'last three months',
                      '15552000' => 'last six months',
                      '31104000' => 'last year',
                      'select_form_order' =>
                      ['-1','86400','604800','2592000','7776000',
		      '15552000','31104000']));
       
    $r->print(&Apache::loncommon::bodytag("Selecting a Course"));
    my %lt=&Apache::lonlocal::texthash(
				       'cac' => 'Course Activity',
				       'cde' => 'Course Description',
				       'cdo' => 'Course Domain');
    my ($name_code,$name_input);
    if (defined($ENV{'form.cnameelement'}) && $ENV{'form.cnameelement'} ne '') {
	$name_code = "opener.document.$ENV{'form.form'}.$ENV{'form.cnameelement'}.value=cdesc;";
	$name_input ='<input type="hidden" name="cnameelement" value="'.
	    $ENV{'form.cnameelement'}.'" />';
    }
    $r->print(<<ENDSCRIPT);
<script>
function gochoose(cname,cdom,cdesc) {
    $name_code
    opener.document.$ENV{"form.form"}.$ENV{'form.cnumelement'}.value=cname;
    var slct=opener.document.$ENV{'form.form'}.$ENV{'form.cdomelement'};
    var i;
    for (i=0;i<slct.length;i++) {
        if (slct.options[i].value==cdom) { slct.selectedIndex=i; }
    }
    self.close();
}
</script>
<form method="post">
<input type="hidden" name="cnumelement" value="$ENV{'form.cnumelement'}" />
<input type="hidden" name="cdomelement" value="$ENV{'form.cdomelement'}" />
$name_input
<input type="hidden" name="form" value="$ENV{'form.form'}" />

$lt{'cac'}: $sincefilterform
<br />
$lt{'cdo'}: $domainselectform
<br />
$lt{'cde'}:
<input type="text" name="descriptfilter" size="40" value="$listdescript" />
<p><input type="submit" name="gosearch" value="Search" /></p>
</form>
<hr />
ENDSCRIPT
# ---------------------------------------------------------------- Get the data
    if ($ENV{'form.gosearch'}) {
        $r->print(&mt('Searching').' ...<br />&nbsp;<br />');
        $r->rflush();
	unless ($descriptfilter) { $descriptfilter='.'; }
        my $timefilter=($sincefilter==-1?1:time-$sincefilter);
	my %courses=
	    &Apache::lonnet::courseiddump
	    ($domainfilter,$descriptfilter,$timefilter);
	$r->print('<form>');
	my %by_descrip;
	foreach my $course (keys %courses) {
            my $descr;
            if ($courses{$course} =~ m/^([^:]*):/) {
                $descr = &Apache::lonnet::unescape($1);
            } else {
                $descr = &Apache::lonnet::unescape($courses{$course});
            }
            $by_descrip{lc($descr)}=$course;
	}
	foreach my $description (sort(keys(%by_descrip))) {
	    my $course=$by_descrip{$description};
	    my $cleandesc=&HTML::Entities::encode($description,'<>&"');
	    $cleandesc=~s/'/\\'/g;
	    my ($cdom,$cnum)=split(/\_/,$course);
	    $r->print('<input type="button" value="Select" onClick="gochoose('.
		  "'".$cnum."','".$cdom."','".$cleandesc."')".'" /> '.$description.' ('.
		  ($Apache::lonnet::domaindescription{$cdom}?
                   $Apache::lonnet::domaindescription{$cdom}:$cdom).
                 ")<br />\n");
	}
	if (!%courses) { $r->print(&mt('None found')); }
        $r->print('</form>');
    }
    $r->print('</body></html>');
    return OK;
} 

1;
__END__
