# automated enrollment configuration handler
# $Id: lonpopulate.pm,v 1.20 2004/09/13 16:21:30 raeburn Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
package Apache::lonpopulate;

use strict;
use lib qw(/home/httpd/lib/perl);
use Apache::lonnet;
use Apache::loncommon;
use Apache::lonhtmlcommon;
use Apache::lonlocal;
use Apache::loncoursedata;
use Apache::Constants qw(:common :http REDIRECT);
use Time::Local;
use LONCAPA::Enrollment;

###############################################################
sub header {
    my $bodytag=&Apache::loncommon::bodytag('Classlist Manager');
    return(<<ENDHEAD);
<html>
<head>
<title>LON-CAPA Classlist Manager</title>
</head>
$bodytag
ENDHEAD
}

###############################################################

sub choose_header {
    my $action = shift;
    my $bodytag=&Apache::loncommon::bodytag('Classlist Manager');
    my $scripttag = qq|
<script language='javascript' type='text/javascript'>
<!--
function process(calling,numauto,nummanual,numlock,numunlock) {
 var checker = 1
 var rad1 = 0
 var rad2 = 0
 var formName = document.forms.enter
 if (calling == "viewclass") {
     formName = document.forms.studentform
 }
 formName.action.value = calling
 if (calling == "chgsettings") {
   for (var j=0; j<formName.autoadds.length; j++) {
       if (formName.autoadds[j].checked) {
           rad1 = 1
       }
   }
   for (var k=0; k<formName.autodrops.length; k++) {
       if (formName.autodrops[k].checked) {
           rad2 = 1
       }
   }
   if (rad1 == 0) {
       alert("You must select either 'Enable' or 'Disable' for nightly additions based on classlist changes")
       checker = 0
   }
   if (rad2 == 0) {
       alert("You must select either 'Enable' or 'Disable' for nightly removals based on classlist changes")
       checker = 0
   }
 }
 if (calling == "updatenow") {
     var enrolldis
     var unenrolldis
     for (var j=0; j<formName.updateadds.length; j++) {
         if (formName.updateadds[j].value == 0) {
             enrolldis = j
         }
         if (formName.updateadds[j].checked) {
             rad1 = 1
         }
     }
     for (var k=0; k<formName.updatedrops.length; k++) {
         if (formName.updatedrops[k].value == 0) {
             unenrolldis = k 
         }
         if (formName.updatedrops[k].checked) {
             rad2 = 1
         }
     }
     if (rad1 == 0) {
         alert("You must select either 'Yes' or 'No' for immediate addition of newly registered students to the roster")
         checker = 0
     }
     if (rad2 == 0) {
         alert("You must select either 'Yes' or 'No' for immediate removal of unregistered students from the roster")
         checker = 0
     }
     if (formName.updatedrops[unenrolldis].checked && formName.updateadds[enrolldis].checked ) {
         alert("You have selected 'No' for both addition and removal of students\\n in the Registrar's classlist but not in your LON-CAPA course.  \\nHence there is no update to carry out")
         checker = 0
     }
 }
 if (calling == "viewclass") {
     var totcheck = 0
     var numchk = 0
     if (numauto > 0) {
         numchk = countChecked(document.studentform.chgauto);
         totcheck = totcheck + numchk
     }
     if (nummanual > 0) {
         numchk = countChecked(document.studentform.chgmanual);
         totcheck = totcheck + numchk
     }
     if (numlock > 0) {
         numchk = countChecked(document.studentform.lockchg);
         totcheck = totcheck + numchk
     }
     if (numunlock > 0) {
         numchk = countChecked(document.studentform.unlockchg);
         totcheck = totcheck + numchk
     }
     if (totcheck > 0) {
        document.forms.studentform.state.value = "process";
     }
     if (totcheck == 0) {
        alert("You must check at least one checkbox, before proceeding to the next page")
        checker = 0
     }
 } 
 if (checker == 1) {  
     formName.submit();
 }
}
|;
    if ($action eq 'viewclass') {
        $scripttag .= qq|
function checkAll(field) {
    if (field.length > 0) {
        for (i = 0; i < field.length; i++) {
            field[i].checked = true ;
        }
    } else {
        field.checked = true
    }
}

function uncheckAll(field) {
    if (field.length > 0) {
        for (i = 0; i < field.length; i++) {
            field[i].checked = false ;
        }
    } else {
        field.checked = false ;
    }
}

function countChecked(field) {
    var count = 0;
    if (field.length > 0) {
        for (var i=0; i<field.length; i++) {
            if (field[i].checked == true) {
                count ++
            }
        }
    } else {
        if (field.checked == true) {
            count ++
        }
    }
    return count
}

|;
    }
    $scripttag .= qq|
// End hiding -->
</script>
|;
    return(<<ENDHEAD);
<html>
<head>
<title>LON-CAPA Classlist Manager</title>
$scripttag
$bodytag
ENDHEAD
}

sub print_mainbox {
    my ($r,$tasklongref,$realm,$reply) = @_;
    my $action = "information";
    if ( exists($ENV{'form.action'}) ) {
        $action = $ENV{'form.action'};
    }
    my $page = '';
    if ($action eq "information") {
        $page = "<b>Automated Enrollment</b>";
    } else {
        $page =  '<a href="/adm/populate">Automated Enrollment</a>';
        if ($reply) {
            if ($action eq "newcross") {
                $action = "crosslist";
            } elsif ($action eq "newsections") {
                $action = "sections"; 
            }
            $page .= "-&gt; <a href=\"/adm/populate?action=$action\">".$$tasklongref{$action}."</a> -&gt; <b>result</b>";
        } else {
            $page .=  " -&gt; <b>".$$tasklongref{$action}."</b>"; 
        }
    }
    $r->print(<<ENDTHIS);
<table width="100%" border="0" cellpadding="0" cellspacing="0">
 <tr>
  <td bgcolor="#CCCCFF"> 
   <font size="2"><a href="/adm/menu">$realm</a> -&gt; <a href="/adm/dropadd">Enrollment Manager</a> -&gt; $page</font><br/>
  </td>
  <td align="right" bgcolor="#CCCCFF" valign="top">
   <font size="+1">Automated Enrollment Manager&nbsp;</font>
  </td>
 </tr>
</table>
<table width="100%" border="0" cellpadding="0" cellspacing="0">
 <tr>
ENDTHIS
}

sub print_navmenu {
    my ($r,$tasksref,$tasklongref) = @_;
    my $action = "information";
    if (exists($ENV{'form.action'}) ) {
        $action = $ENV{'form.action'};
    }
    $r->print(<<ENDONE);
  <td width="10" valign="top" bgcolor="#DDFFFF">&nbsp;</td>
  <td width="20%" valign="top" bgcolor="#DDFFFF">
   <br/>
ENDONE
    foreach my $task (@{$tasksref}) {
        if ($task eq $action) {
            $r->print(" 
   <p>
   <font color=\"#999999\">
    <b>$$tasklongref{$task}</b><br/>
   </font>
   </p>"); 
        } else {
            $r->print("
   <p>
   <font color=\"#004263\">
    <b><a href=\"/adm/populate?action=$task\">$$tasklongref{$task}</a></b><br/>
   </font> 
   </p>");

        }
    }
    $r->print("
  <p>&nbsp;</p>
  </td>
  <td width=\"10\" valign=\"top\" bgcolor=\"#CCCCFF\">&nbsp;</td>
  <td width=\"10\" valign=\"top\" bgcolor=\"#FFFFFF\">&nbsp;</td>
  <td bgcolor=\"#ffffff\" valign=\"top\">");
}

###############################################################

sub print_main_frame {
  my ($r,$realm,$dom,$crs,$tasktitleref) = @_;
  my $action = "information";
  if (exists($ENV{'form.action'}) ) {
      $action = $ENV{'form.action'};
  }

# Get course settings
  my %enrollvar;
  my @bgcolors=("#eeeeee","#cccccc");
  my %settings = &Apache::lonnet::dump('environment',$dom,$crs);
  foreach my $item (keys %settings) {
      if ($item =~ m/^internal\.(.+)$/) {
          $enrollvar{$1} = $settings{$item};
      } elsif ($item =~ /^default_enrollment_(start|end)_date$/) {
          $enrollvar{$item} = $settings{$item};
      }
  }

  if ($action eq "information") {
      $r->print(<<ENDONE);
          <br/><table border='0' width='100%'>
            <tr>
              <td>&nbsp;</td>
              <td><b>Use the menu on the left to choose an enrollment management task.</b><br/><br/></td>
            <tr>
              <td>&nbsp;</td>
              <td>Use <i>"Automated adds/drops"</i> to enable or disable automatic nightly adds or drops in your LON-CAPA course based on institutional enrollment information.</td>
            </tr>
            <tr>
              <td>&nbsp;</td>
              <td>Use <i>"Change enrollment dates"</i> to change the date of first automated enrollment and/or the date of last automated enrollment for registered students.</td>
            </tr>
            <tr>
              <td>&nbsp;</td>
              <td>Use <i>"Change access dates"</i> to change the default start and/or end dates for student roles created by automated enrollment.</td>
            </tr>
            <tr>
              <td>&nbsp;</td>
              <td>Use <i>"Notification of changes"</i> to enable or disable notification of enrollment changes and to add or remove course coordinators from the recipient list.</td>
            </tr>
            <tr>
              <td>&nbsp;</td>
              <td>Use <i>"Change crosslisting"</i> to include or exclude enrollment from crosslisted classes.</td>
            </tr>
            <tr>
              <td>&nbsp;</td>
              <td>Use <i>"Section settings"</i> to make changes to the choice of sections included for enrollment in your LON-CAPA course.</td>
            </tr>
              <td>&nbsp;</td>
              <td>Use <i>"Student photos"</i> to enable or disable automatic upload of student photos to your course.</td>
            </tr>
            <tr>
              <td>&nbsp;</td>
              <td>Use <i>"Update roster now"</i> to add and/or drop students from your course based on the <b>most current</b> institutional classlist information.</td>
            </tr>
            <tr>
              <td>&nbsp;</td>
              <td>Use <i>"View students and change type"</i> to display the current course roster, and (optionally) change enrollment type for selected students from 'auto' to 'manual' and vice versa.</td>
            </tr>
            <tr>
             <td colspan='2'>&nbsp;</td>
            </tr>
            <tr>
             <td>&nbsp;</td>
             <td><b>Note: if automated adds and/or drops are enabled, the nightly enrollment update will ONLY occur once the first enrollment date has been reached.</b></td>
            </tr>
          </table>
ENDONE
  } elsif ($action eq "chgsettings") {
      my @autosets = ("OFF","ON");
      $r->print(<<ENDTWO);
                  <form name="enter" method="post"><br/>
		  <table width="100%" border="0" cellpadding="2" cellspacing="2">
		   <tr>
		    <td align="left"><b>$$tasktitleref{$action}</b><br/>
		       Currently: Nightly adds: <i>$autosets[$enrollvar{autoadds}]</i>, Nightly drops: <i>$autosets[$enrollvar{autodrops}]</i>
		    </td>
                   </tr>
		  </table>
		  <table width="100%" border="0" cellpadding="3" cellspacing="3">
		    <tr>
		     <td>
		         Additions based on classlist changes:&nbsp;&nbsp;
ENDTWO
      if ($enrollvar{autoadds}) {
          $r->print("
			    <input type=\"radio\" name=\"autoadds\" value=\"1\" checked=\"true\" />&nbsp;Enable&nbsp;&nbsp;&nbsp;
			    <input type=\"radio\" name=\"autoadds\" value=\"0\" />&nbsp;Disable
         ");
      } else {
          $r->print("
                            <input type=\"radio\" name=\"autoadds\" value=\"1\" />&nbsp;Enable&nbsp;&nbsp;&nbsp;
                            <input type=\"radio\" name=\"autoadds\" value=\"0\" checked=\"true\" />&nbsp;Disable
         ");
      }
      $r->print("
              </td>
             </tr>
             <tr>
              <td>
 	       Removals based on classlist changes:&nbsp;&nbsp;");
      if ($enrollvar{autodrops}) {
          $r->print("
                <input type=\"radio\" name=\"autodrops\" value=\"1\" checked=\"true\" />&nbsp;Enable&nbsp;&nbsp;&nbsp;
                <input type=\"radio\" name=\"autodrops\" value=\"0\" />&nbsp;Disable");
      } else {
          $r->print("
                <input type=\"radio\" name=\"autodrops\" value=\"1\" />&nbsp;Enable&nbsp;&nbsp;&nbsp;
                <input type=\"radio\" name=\"autodrops\" value=\"0\" checked=\"true\" />&nbsp;Disable");
      }
      $r->print("
              </td>
             </tr>
             <tr>
              <td>
               <font color=\"#888888\">
Note: Any students added manually by course coordinators using the Enrollment Manager will be unaffected by the nightly removal process if you choose to enable it.
               </font>
              </td>
             </tr>
             <tr>
              <td align=\"right\">
               <input type=\"button\" name=\"chgsettings\" value=\"Go\" onClick=\"process('chgsettings')\" />
	      </td>
             </tr>
	    </table>
            <input type=\"hidden\" name=\"action\" value=\"$action\">
            <input type=\"hidden\" name=\"state\" value=\"process\">
            </form>
      ");
  } elsif ($action eq "setdates") {
      my ($start_table,$end_table) = &date_setting_table($enrollvar{autostart},$enrollvar{autoend},$action);
      my $oldstartshow = '';
      my $oldendshow = '';
      if ( defined($enrollvar{autostart}) ) {
          $oldstartshow = &Apache::lonlocal::locallocaltime($enrollvar{autostart});
      }
      if ( defined($enrollvar{autoend}) ) {
          $oldendshow = &Apache::lonlocal::locallocaltime($enrollvar{autoend});
          if ($enrollvar{autoend} == 0) {
              $oldendshow = "No ending date";
          }
      }
      my $dateshow;
      if ( ($oldendshow eq '') && ($oldstartshow eq '') ) {
         $dateshow = "<br/><font size='+1'>Warning</font>. Currently <b>NO</b> first enrollment or last enrollment dates are set. You <b>must</b> use this menu to set a start date and an end date if you plan to utilise automated adds and/or drops in this course.\n";
      } else {
         $dateshow = "Currently: First enrollment: <b><i>$oldstartshow</i></b>, Last enrollment: <b><i>$oldendshow</i></b>\n";
      }
      $r->print(<<ENDTWO);
                  <form name="enter" method="post"><br/>              
                  <table width="100%" border="0" cellpadding="2" cellspacing="2">
	           <tr>
	            <td align="left"><b>$$tasktitleref{$action}</b><br/><br/>
                     $dateshow
	            </td>
                   </tr>
	          </table>
                  <table width="100%" border="0" cellpadding="3" cellspacing="3">
                   <tr>
                    <td align="left" colspan="2">
                     <table border="0" cellspacing="0" cellpadding="2">
                      <tr>
                       <td colspan="3">
                        <i>Set date of first automated enrollment for registered students</i>
                       </td>
                      </tr>
                      <tr>
                       <td>$start_table
                       </td>
                      </tr>
                     </table>
                    </td>
                   </tr>
                   <tr>
                    <td colspan="2"><font color="#888888">If automated adds and/or drops are enabled, then your class roster will be automatically updated nightly, once the first enrollment date has been reached. Prior to this date, the class roster will only contain students you have added directly using the standard LON-CAPA enrollment tools</font></td>
                   </tr>
                   <tr>
                    <td align="left" colspan="2">
                     <table border="0' cellspacing="0" cellpadding="2">
                      <tr>
                       <td colspan="3">
                        <i>Set date of last automated enrollment for registered students</i>
                       </td>
                      </tr>
                      <tr>
                       <td>$end_table
                       </td>
                      </tr>
                     </table>
                    </td>
                   </tr>
                   <tr>
                    <td colspan="2"><font color="#888888">If automated adds and/or drops are enabled, then your class roster will be automatically updated nightly, until the last enrollment date has been reached.</font></td>
                   </tr>
                  </table>
                  <table width="100%">
                   <tr>
                    <td align="right">
                      <input type="button" name="setdates" value="Go" onClick="process('setdates')" />
                    </td>
	           </tr>
                  </table>
                  <input type="hidden" name="action" value="$action">
                  <input type="hidden" name="state" value="process">
                  </form>
ENDTWO
  } elsif ($action eq "setaccess") {
      &print_accessdate_table($r,\%enrollvar,$tasktitleref,$action);
      $r->print(<<ENDTWO);
                  <table width="100%">
                   <tr>
                    <td align="right">
                      <input type="button" name="$action" value="Go" onClick="process('$action')" />
                    </td>
                   </tr>
                  </table>
                  <input type="hidden" name="action" value="$action">
                  <input type="hidden" name="state" value="process">
                  </form>
ENDTWO
  } elsif ($action eq "notify") {
      my @notified = ();
      if ($enrollvar{notifylist} =~ m/,/) {
          @notified = split/,/,$enrollvar{notifylist};
      } else {
          @notified = $enrollvar{notifylist};
      }
      my $notifycount = 0;
      foreach  (@notified) {
        unless ($_ eq '') { $notifycount ++; } 
      }
      my $noteset = '';
      if ($notifycount) {
          $noteset = "ON";
      } else {
          $noteset = "OFF";
      }
    
      $r->print("
                  <form name=\"enter\" method=\"post\"><br/>
                   <table width=\"100%\" border=\"0\" cellpadding=\"6\" cellspacing=\"0\">
                    <tr>
                     <td align=\"left\"><b>$$tasktitleref{$action}</b><br/>
                      Currently: Notification: $noteset
                     </td>
                    </tr>
                   </table>
                   <table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
                    <tr>
                     <td>
                      Notification of LON-CAPA course roster changes resulting from nightly automated enrollment process?
      ");
      if ($notifycount) {
          $r->print("
                        <input type=\"radio\" name=\"notify\" value=\"1\" checked=\"true\" />&nbsp;Yes&nbsp;&nbsp;&nbsp;
                        <input type=\"radio\" name=\"notify\" value=\"0\" />&nbsp;No
          ");
      } else {
          $r->print("
                        <input type=\"radio\" name=\"notify\" value=\"1\" />&nbsp;Yes&nbsp;&nbsp;&nbsp;
                        <input type=\"radio\" name=\"notify\" value=\"0\" checked=\"true\" />&nbsp;No
          ");
      }
      $r->print("
              </td>
             </tr>
      ");
      my @coursepersonnel = &Apache::lonnet::getkeys('nohist_userroles',$dom,$crs);
      my @ccs = ();
      my @oldccs = ();
      my %pname = ();
      my %notifystate = ();
      foreach (@coursepersonnel) {
          my @roleinfo = split/:/,$_;
          if ($roleinfo[0] eq 'cc')  {
              unless (grep/^$roleinfo[1]\@$roleinfo[2]/,@ccs) {
                  my $active_cc = &LONCAPA::Enrollment::check_user_status($roleinfo[2],$roleinfo[1],$dom,$crs,'cc');
                  if ($active_cc eq 'ok') {
                      push @ccs, "$roleinfo[1]\@$roleinfo[2]";
                      $pname{"$roleinfo[1]\@$roleinfo[2]"} = &Apache::loncommon::plainname($roleinfo[1],$roleinfo[2]);
                      if (grep/^$roleinfo[1]\@$roleinfo[2]$/,@notified) {
                          $notifystate{"$roleinfo[1]\@$roleinfo[2]"} = 1;
                      } else {
                          $notifystate{"$roleinfo[1]\@$roleinfo[2]"} = 0;
                      }
                  } else {
                      push @oldccs, "$roleinfo[1]\@$roleinfo[2]";
                  }
              }
          }
      }
      if ($notifycount > 0) {
          foreach my $person (@notified) {
              unless ( (grep/^$person$/,@ccs) || ($person eq '') || (grep/^$person$/,@oldccs) ) {
                  push @ccs, $person;
                  my ($puname,$pdom) = split/\@/,$person;
                  $pname{$person} =  &Apache::loncommon::plainname($puname,$pdom);
                  $notifystate{$person} = 1;
              }
          }
      }
      my $viewer = $ENV{'user.name'}.'@'.$ENV{'user.domain'};
      unless ( (grep/^$viewer$/,@ccs) || ($viewer eq '') )  {
          push @ccs,$viewer;
          $pname{$viewer} =  &Apache::loncommon::plainname($ENV{'user.name'},$ENV{'user.domain'});
          $notifystate{$viewer} = 0;
      }

      my $notifyshow = @ccs;

      if (@ccs > 0) {
          @ccs = sort @ccs;
          $r->print("
             <tr>
               <td>The table below contains a list of course coordinators in this course. Uncheck the 'Notification?' checkbox for each person who is to be removed from the list of recipients who are currently informed of roster changes whenever any adds or drops occur during the nightly enrollment check. To include individuals who are not currently recipients, simply check the 'Notification?' checkbox. Click 'Go' to store your changes.
              </td>
             </tr>
             <tr>
              <td>
               <table border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
                <tr bgcolor=\"#CCCCFF\">
                 <td><b>Name</b></td>
                 <td><b>username</b></td>
                 <td><b>Current status</b></td>
                 <td><b>Notification?</b></td>
                </tr>
          ");
          for (my $i=0; $i<@ccs; $i++) {
              my $colflag = $i%2;
              $r->print("<tr bgcolor=\"$bgcolors[$colflag]\" align=\"left\">
                 <td>$pname{$ccs[$i]}</td>
                 <td><input type=\"hidden\" name=\"notifyname_$i\" value=\"$ccs[$i]\" />$ccs[$i]</td>
                 <td>
              ");
              if ($notifystate{$ccs[$i]} == 1) {
                  $r->print("Notification active");
              } else {
                  $r->print("Notification inactive");
              }
              if ($notifystate{$ccs[$i]} == 1) {
                  $r->print("</td><td><input type=\"checkbox\" name=\"note_$i\" checked=\"true\" /></td></tr>");
              } else {
                  $r->print("</td><td><input type=\"checkbox\" name=\"note_$i\" /></td></tr>");
              }
          }
          $r->print("
               </table>
               <br/>
               <table width=\"100%\" border=\"0\" cellspadding=\"2\" cellspacing=\"2\">
                <tr>
                 <td align=\"right\">
                  <input type=\"button\" name=\"notifyset\" value=\"Go\" onClick=\"process('notify')\" />
                 </td>
                </tr>
               </table>
              </td>
             </tr>
          ");
      } else {
          $r->print("
             <tr>
              <td>
             No course coordinators found.
            </td>
           </tr>");
      }
      $r->print("
          </table>
          <input type=\"hidden\" name=\"notifyshow\" value=\"$notifyshow\">
          <input type=\"hidden\" name=\"action\" value=\"$action\">
          <input type=\"hidden\" name=\"state\" value=\"process\">
          </form>
      ");
  } elsif ($action eq "crosslist") {
      my @xlists = ();
      if ($enrollvar{crosslistings} =~ m/,/) {
          @xlists = split/,/,$enrollvar{crosslistings};
      } elsif ($enrollvar{crosslistings} ne '') {
          $xlists[0] = $enrollvar{crosslistings};
      }
      my $cross_str = @xlists;
      $r->print("
            <form name=\"enter\" method=\"post\"><br/>
            <table width=\"100%\" border=\"0\" cellpadding=\"2\" cellspacing=\"2\">
             <tr>
              <td align=\"left\"><b>$$tasktitleref{$action}</b><br/>
      ");
      if ($cross_str > 0) {
          $r->print("
                Currently, this LON-CAPA course is crosslisted with $cross_str course section(s).  Students enrolling in these course sections will be automatically added to the class roster for the course, if you have chosen to enable a nightly automated enrollment update.  For each crosslisting, leave the checkbox checked if you want registered students in that course to be included in the student roster for LON-CAPA course: <b>$realm ($enrollvar{coursecode})</b>; otherwise uncheck it. If you wish to change the section/group ID assigned in your LON-CAPA course for a crosslisted course, enter the new section/group ID in the appropriate textbox. The LON-CAPA section/group ID can be left (or set to) empty, if you do not wish to tie a section/group ID to this crosslisting. If you wish to add new crosslisted courses, enter the number of new courses to add in the textbox at the bottom of the page. You will provide information about each of the new crosslistings on a subsequent page.  Click 'Go' to store your changes.
              </td>
             </tr>
            </table>
            <br/>
            <table border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
             <tr>
              <td>
               <table border=\"0\" width=\"100%\" cellspacing=\"3\" cellpadding=\"3\">
                <tr bgcolor=\"#CCCCFF\" align=\"left\">
                 <td><b>Enrollment?</b></td>
                 <td><b>Crosslisted course</b></td>
                 <td><b>LON-CAPA section/groupID</b></td>
                </tr>
          ");
          for (my $i=0; $i<@xlists; $i++) {
              my $xl = '&nbsp;';
              my $gp = '&nbsp;';
              if ($xlists[$i] =~ /(\w+):?(.*)$/) {
                  $xl = $1;
                  $gp = $2;
              }               
              my $colflag = $i%2;
              $r->print("
                <tr bgcolor=\"$bgcolors[$colflag]\" align=\"left\">
                 <td><input type=\"checkbox\" name=\"cross_$i\" checked=\"true\"/></td>
                 <td>$xl</td>
                 <td><input type =\"text\" size=\"10\" name=\"gp_$i\" value=\"$gp\"></td>
                </tr> 
              ");
          }
          $r->print("  </table>
              </td>
             </tr>
            </table>
          ");
      }
      else {
          $r->print("
                Currently no crosslisted courses are recorded for $enrollvar{coursecode}.
              </td>
             </tr>
            </table>
          ");
      }
      $r->print("   
            <br/>
            <table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
             <tr>
              <td align=\"left\">
               <b>Add new crosslistings.</b><br/>
               Number of new crosslistings to add:&nbsp;&nbsp;<input type=\"text\" size=\"2\" name=\"numcross\" value=\"0\" />
              </td>
             </tr>
            </table>
            <br/>
            <table width=\"100%\" border=\"0\" cellspadding=\"2\" cellspacing=\"2\">
             <tr>
              <td align=\"right\">
               <input type=\"button\" name=\"crosslist\" value=\"Go\" onClick=\"process('crosslist')\" />
              </td>
             </tr>
            </table>
            <input type=\"hidden\" name=\"action\" value=\"$action\">
            <input type=\"hidden\" name=\"state\" value=\"process\">
            </form> 
      ");
  } elsif ($action eq "sections") {
      my @sections = ();
      @sections = &Apache::lonnet::auto_get_sections($crs,$dom,$enrollvar{coursecode});
      my @storedsections = ();
      my @currsections = ();
      my %sec_id = ();
      if ($enrollvar{sectionnums} =~ m/,/) {
          @storedsections = split/,/,$enrollvar{sectionnums};
      } else {
          $storedsections[0] = $enrollvar{sectionnums};
      }
      foreach (@storedsections) {
          if ($_ =~ m/^(\w+):(\w*)$/) {
              push @currsections, $1;
              $sec_id{$1} = $2;
          }
      }
      if (@sections > 0) {
          my $secshow = @sections;
          $r->print("
            <form name=\"enter\" method=\"post\"><br/>
            <table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
             <tr>
              <td align=\"left\">
               <b>$$tasktitleref{$action}</b><br/>
               Your institution's course catalog includes $secshow sections for course code: $enrollvar{coursecode}.
              </td>
             </tr>
             <tr>
              <td>For each section, check the checkbox if you want registered students in that section to be included in the student roster for LON-CAPA course: <b>$realm ($enrollvar{coursecode})</b>; otherwise uncheck it. If you want to change the section/group designation used for this section in LON-CAPA, delete the current value in the LON-CAPA section/group textbox and enter the new value. The LON-CAPA section/group ID can be left (or set to) empty, if you do not wish to tie a section/group ID to this section. To add a new section, check the 'Enrollment in this course?' checkbox, and enter the desired LON-CAPA section/groupID in the appropriate textbox. Click 'Go' to store
your changes.</td>
             </tr>
            </table>
            <br/>
            <table border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
             <tr bgcolor=\"#CCCCFF\">
              <td><b>Section</b></td>
              <td><b>Current status</b></td>              
              <td><b>LON-CAPA section/group ID</b></td>
              <td><b>Enrollment in this course?</b></td>
             </tr>
          ");
          for (my $i=0; $i<@sections; $i++) {
              my $colflag = $i%2;
              my $shrflag = 0;
              $r->print("<tr bgcolor=\"$bgcolors[$colflag]\" align=\"left\">
                   <td>$sections[$i]<input type=\"hidden\" name=\"secnum_$i\" value=\"$sections[$i]\" /></td>
                   <td>
              ");
              if (grep/^$sections[$i]$/,@currsections) {
                  $r->print("Enrollment active");
                  $shrflag = 1;
              } else {
                  $r->print("Enrollment inactive");
              }
              if ($shrflag) {
                  $r->print("</td><td><input type=\"text\" size=\"10\" name=\"loncapasec_$i\" value=\"$sec_id{$sections[$i]}\"></td><td><input type=\"checkbox\" name=\"sec_$i\" checked=\"true\" /></td></tr>"); 
              } else {
                  $r->print("</td><td><input type=\"text\" size=\"10\" name=\"loncapasec_$i\" value=\"\"><td><input type=\"checkbox\" name=\"sec_$i\" /></td></tr>");
              }
          }
          $r->print("
            </table>
            <br/>
            <table width=\"100%\" border=\"0\" cellspacing=\"3\" cellpadding=\"3\">
             <tr> 
              <td align=\"right\">
               <input type=\"hidden\" name=\"secshow\" value=\"$secshow\" />
               <input type=\"button\" name=\"sections\" value=\"Go\" onClick=\"process('sections')\" />
              </td>
             </tr>
            </table>
            <input type=\"hidden\" name=\"action\" value=\"$action\">
            <input type=\"hidden\" name=\"state\" value=\"process\">
            </form>
          ");
      } else {
          $r->print("
            <form name=\"enter\" method=\"post\"><br/>
            <table width=\"100%\" border=\"0\" cellpadding=\"2\" cellspacing=\"2\">
             <tr>
              <td align=\"left\"><b>$$tasktitleref{$action}</b><br/>
          ");
          if (@currsections) {
              my $secshow = @currsections;
              $r->print("
                Currently, this LON-CAPA course incorporates enrollment from $secshow sections.  Students enrolling in any of these course sections will be automatically added to the class roster for the course, if you have chosen to enable a nightly automated enrollment update. For each section, uncheck the checkbox if you want registered students in that section to cease being included in the student roster for LON-CAPA course: <b>$realm ($enrollvar{coursecode})</b>; otherwise leave it checked. If you want to change the section/group designation used for this section in LON-CAPA, delete the current value in the LON-CAPA section/groupID textbox and enter the new value.  If you wish to add new course section, enter the number of new sections to add in the textbox at the bottom of the page. You will provide information about each of the new sections on a subsequent page. Click 'Go' to store your changes.
              </td>
             </tr>
            </table>
            <br/>
            <table border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
             <tr>
              <td>
               <table border=\"0\" width=\"100%\" cellspacing=\"3\" cellpadding=\"3\">
                <tr bgcolor=\"#CCCCFF\">
                 <td><b>Enrollment?</b></td>
                 <td><b>Section</b></td>
                 <td><b>LON-CAPA section/groupID</b></td>
                </tr>
              ");
              for (my $j=0; $j<@currsections; $j++) {
                  my $colflag = $j%2;
                  $r->print("
                <tr bgcolor=\"$bgcolors[$colflag]\" align=\"left\">
                 <td><input type=\"checkbox\" name=\"sec_$j\" checked=\"true\"/></td>
                 <td>$currsections[$j]</td>
                 <td><input type=\"text\" name=\"secgp_$j\" size=\"10\" value=\"$sec_id{$currsections[$j]}\"></td>
                </tr> 
                  ");
              }
              $r->print("  </table>
              </td>
             </tr>
            </table>
              ");
          } else {
              $r->print("
                Currently no sections of $realm ($enrollvar{coursecode}) are contributing enrollment to the LON-CAPA class roster.
              </td>
             </tr>
            </table>
              ");
          }
          $r->print("   
            <br/>
            <table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
             <tr>
              <td align=\"left\">
               <b>Add enrollment from additional sections.</b><br/>
               Number of new sections to add:&nbsp;&nbsp;<input type=\"text\" size=\"2\" name=\"numsec\" value=\"0\" />
              </td>
             </tr>
            </table>
            <br/>
            <table width=\"100%\" border=\"0\" cellspadding=\"2\" cellspacing=\"2\">
             <tr>
              <td align=\"right\">
               <input type=\"button\" name=\"sections\" value=\"Go\" onClick=\"process('sections')\" />
              </td>
             </tr>
            </table>
            <input type=\"hidden\" name=\"action\" value=\"$action\">
            <input type=\"hidden\" name=\"state\" value=\"process\">
            </form> 
          ");
      }
  } elsif ($action eq "photos") {
      my @photosets = ("OFF","ON");
      $r->print("
                  <form name=\"enter\" method=\"post\"><br/>
                  <table width=\"100%\" border=\"0\" cellpadding=\"2\" cellspacing=\"2\">
                   <tr>
                    <td align=\"left\"><b>$$tasktitleref{$action}</b><br/>
                       Currently: Student photo import:  <i>$photosets[$enrollvar{showphotos}]</i>
                    </td>
                   </tr>
                  </table>
                  <table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
                    <tr>
                     <td>
                         Automatic import of student photos from institutional data repository:&nbsp;&nbsp;

      ");
      if ($enrollvar{showphotos}) {
          $r->print("
                        <input type=\"radio\" name=\"showphotos\" value=\"1\" checked=\"true\" />&nbsp;Yes&nbsp;&nbsp;&nbsp;
                        <input type=\"radio\" name=\"showphotos\" value=\"0\" />&nbsp;No
          ");
      } else {
          $r->print("
                        <input type=\"radio\" name=\"showphotos\" value=\"1\" />&nbsp;Yes&nbsp;&nbsp;&nbsp;
                        <input type=\"radio\" name=\"showphotos\" value=\"0\" checked=\"true\" />&nbsp;No
          ");
      }
      $r->print("
                     </td>
                    </tr>
                    <tr>
                     <td>
                      <font color=\"#888888\">
Note: if you enable automatic import of student photos, your course will automatically have access to photos stored by your institution for all officially registered students, via a conduit established by your LON-CAPA domain coordinator.  
                      </font>
                     </td>
                    </tr>
                    <tr>
                     </td>&nbsp;</td>
                    </tr>
                    <tr>
                     <td align=\"right\">
                      <input type=\"button\" name=\"showphotos\" value=\"Go\" onClick=\"process('photos')\">
                     </td>
                    </tr>
                   </table>
                   <input type=\"hidden\" name=\"action\" value=\"$action\">
                   <input type=\"hidden\" name=\"state\" value=\"process\">
                   </form>
      ");
  } elsif ($action eq "updatenow") {
      $r->print("
                   <form name=\"enter\" method=\"post\"><br/>
		   <table width=\"100%\" border=\"0\" cellpadding=\"2\" cellspacing=\"2\">
		    <tr>
		     <td align=\"left\"><b>$$tasktitleref{$action}</b>
		     </td>
                    </tr>
		   </table>
		   <table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"3\">
		    <tr>
		     <td>
	              Add any students currently included in institutional classlist(s) but not enrolled in your LON-CAPA course.<br />  
		      <input type=\"radio\" name=\"updateadds\" value=\"1\">&nbsp;Yes&nbsp;<input type=\"radio\" name=\"updateadds\" value=\"0\">&nbsp;No&nbsp;
                     </td>
                    </tr>
                    <tr>
                     <td>
 	              Expire students previously added by nightly enrollment process, but no longer listed in institutional  classlist(s).<br />                  
                      <input type=\"radio\" name=\"updatedrops\" value=\"1\">&nbsp;Yes&nbsp;
                      <input type=\"radio\" name=\"updatedrops\" value=\"0\">&nbsp;No&nbsp;<br/>
                     </td>
                    </tr>
                    <tr>
                     <td><font color=\"#888888\">
Note: Any students previously added manually by course coordinator(s) using either 'Upload classlist CSV file' or 'Enroll a single user' will be unaffected by the removal process.
                      </font>
                     </td>
                    </tr>
                    <tr>
                     <td>
      ");
      &print_accessdate_table($r,\%enrollvar,$tasktitleref,$action);
      $r->print("
                     </td>
                    </tr>
                    <tr>
                     <td align=\"right\">
                      <input type=\"button\" name=\"updatenow\" value=\"Go\" onClick=\"process('updatenow')\" />
                     </td>
                    </tr>
	           </table>
                   <input type=\"hidden\" name=\"action\" value=\"$action\" />
                   <input type=\"hidden\" name=\"state\" value=\"process\" />
                  </form>
      ");
  } elsif ($action eq 'viewclass') {
      $r->print("
                   <form name=\"studentform\" method=\"post\"><br/>
                   <table width=\"100%\" border=\"0\" cellpadding=\"2\" cellspacing=\"2\">
                    <tr>
                     <td align=\"left\"><b>$$tasktitleref{$action}</b>
                     </td>
                    </tr>
                    <tr>
                     <td>Section changes, name changes, and class drops that can be set to occur either automatically or when using the <b>'Update roster now'</b> feature, will affect only those students with an enroll type of <b>'auto'</b>. Students with an enroll type of <b>'manual'</b>, will be converted automatically to the 'auto' type, when they first appear in the institutional classlist for the course - as long as nightly adds are enabled and active, or the update roster utility is used.  Use the 'Lock' checkbox for any manually enrolled students for whom you wish to prevent type conversion. Use the 'Change' checkbox to switch the enroll type from auto to manual, and vice versa. Use the 'Unlock' checkbox for any maually enrolled students for whom you no longer wish to lock the enroll type.  Click the 'Go' button at the end of the page to process your desired changes.</td>
                    </tr>
                    <tr><td>&nbsp;</td></tr> 
                   </table>
                   <table>
                    <tr>
                     <td>
      ");
      if (! exists($ENV{'form.sortby'})) {
          $ENV{'form.sortby'} = 'username';
      }
      if ($ENV{'form.Status'} !~ /^(Any|Expired|Active)$/) {
          $ENV{'form.Status'} = 'Active';
      }
      my $status_select = &Apache::lonhtmlcommon::StatusOptions($ENV{'form.Status'});
#  Get current classlist
      my ($classlist,$keylist)=&Apache::loncoursedata::get_classlist();
      if (! defined($classlist)) {
          $r->print(&mt('There are no students either currently or previously enrolled.').
                      "\n");
      } else {
          $r->print(&mt('Student Status: [_1]',$status_select)."\n");
          $r->print('<input type="submit" value="'.&mt('Update Display').'" />'.
              "\n</p>\n");

          my ($studentcount,$autocount,$manualcount,$lockcount,$unlockcount) = &Apache::londropadd::show_class_list($r,'autoenroll','nothing',$ENV{'form.Status'},$classlist,$keylist);
          $r->print("
                     </td>
                    </tr>
          ");
          if ($studentcount > 0) {
              $r->print("
                    <tr><td><table border=\"0\" cellpadding=\"5\"><tr>
              ");
              my $cellcount = 0;
              if ($autocount > 0) {
                  $cellcount ++;
                  $r->print(<<END);
                       <td><fieldset><legend>&nbsp;<b>Change auto</b></legend><input type="button" value="check all" onclick="javascript:checkAll(document.studentform.chgauto)" />&nbsp;&nbsp;
                       <input type="button" value="uncheck all" onclick="javascript:uncheckAll(document.studentform.chgauto)" /></fieldset></td>
END
              }
              if ($manualcount > 0) {
                  $cellcount ++;
                  $r->print(<<END);
                       <td><fieldset><legend>&nbsp;<b>Change manual</b></legend><input type="button" value="check all" onclick="javascript:checkAll(document.studentform.chgmanual)" />&nbsp;&nbsp;
                       <input type="button" value="uncheck all" onclick="javascript:uncheckAll(document.studentform.chgmanual)" /></fieldset></td>
END
              }
              if ($lockcount > 0) {
                  if ($cellcount == 2) {
                      $r->print("</tr><tr>");
                  }
                  $cellcount ++;
                  $r->print(<<END);
                       <td><fieldset><legend>&nbsp;<b>Lock manual</b></legend><input type="button" value="check all" onclick="javascript:checkAll(document.studentform.lockchg)" />&nbsp;&nbsp;
                       <input type="button" value="uncheck all" onclick="javascript:uncheckAll(document.studentform.lockchg)" /></fieldset></td>
END
              }
              if ($unlockcount > 0) {
                  if ($cellcount == 2) {
                      $r->print("</tr><tr>");
                  }
                  $cellcount ++;
                  $r->print(<<END);
                       <td><fieldset><legend>&nbsp;<b>Unlock manual</b></legend><input type="button" value="check all" onclick="javascript:checkAll(document.studentform.unlockchg)" />&nbsp;&nbsp;
                       <input type="button" value="uncheck all" onclick="javascript:uncheckAll(document.studentform.unlockchg)" /></fieldset></td>
END
              }
              $r->print("
                       </tr>
                      </table>
                     <td>&nbsp;</td>
                    </tr>
                    <tr>
                     <td align=\"right\">
                      <input type=\"button\" name=\"viewclass\" value=\"Go\" onClick=\"process('viewclass','$autocount','$manualcount','$lockcount','$unlockcount')\" />
                     </td>
                    </tr>
              ");
          }
          $r->print("
                   </table>
                   <input type=\"hidden\" name=\"action\" value=\"$action\" />
                   <input type=\"hidden\" name=\"state\" value=\"choose\" />
                  </form>
          ");
      }
  }
}

sub print_accessdate_table {
    my ($r,$enrollvar,$tasktitleref,$action) = @_;
    my ($start_table,$end_table) = &date_setting_table($$enrollvar{'default_enrollment_start_date'},$$enrollvar{'default_enrollment_end_date'},$action);
    my $oldstartshow = '';
    my $oldendshow = '';
    if ( defined($$enrollvar{'default_enrollment_start_date'}) ) {
        $oldstartshow = &Apache::lonlocal::locallocaltime($$enrollvar{'default_enrollment_start_date'});
    }
    if ( defined($$enrollvar{'default_enrollment_end_date'}) ) {
        $oldendshow = &Apache::lonlocal::locallocaltime($$enrollvar{default_enrollment_end_date});
        if ($$enrollvar{'default_enrollment_end_date'} eq '0') {
            $oldendshow = "No ending date";
        }
    }
    my %lt =&Apache::lonlocal::texthash(
         'cuno' => 'Currently NO default first access or last access dates are set.',
         'ifyo' => 'If you do not set a start date and an end date, then student roles for students added by the automated enrollment process will start immediately when the student is added and will never become inactive.',
         'ifyd' => 'If you do not set an access start date and an end date, then student roles for new students added when you click "Go" will become active immediately and will never become inactive.',
         'setf' => 'Set date of first access',
         'setl' => 'Set date of last access',
         'freg' => 'for registered students added via automated enrollment',
         'fnew' => 'for new students added when you update the class roster',
         'ifad'  => 'If automated adds are enabled, then when students are added their student roles will become active on the date set here for first access, and their roles will become inactive on the date set here for last access.  These default access dates will be overridden for specific students if the institutional classlist data supplied to the automatic enrollment process includes entries for the startdate and enddate fields for those students.',
         'ncds'  => 'changing default start and end access dates will affect future enrollments and ALSO currently inactive students (i.e., those for whom access will begin in the future).  To change access dates for currently active students, you should change the access dates via this screen, then use Enrollment manager -> Drop Students to drop the students, and then use Automated Enrollment Manager -> Update roster now to re-enroll them with the new access dates.'

    );
    my $dateshow;
    if ( ($oldendshow eq '') && ($oldstartshow eq '') ) {
       $dateshow = "<br/><font size='+1'>".&mt('Warning')."</font>.&nbsp;".$lt{'cuno'}." ";
       if ($action eq 'setaccess') {
           $dateshow .= $lt{'ifyo'}."\n";
       } elsif ($action eq 'updatenow') {
           $dateshow .= $lt{'ifyd'}."\n";
       }
    } else {
        $dateshow = &mt('Currently: default first access').": <b><i>$oldstartshow</i></b>, ".&mt('default last access').": <b><i>$oldendshow</i></b>\n";
    }
    if ($action eq 'setaccess') {
        $r->print(<<ENDONE);
                <form name="enter" method="post"><br/>
                <table width="100%" border="0" cellpadding="2" cellspacing="2">
                 <tr>
                  <td align="left"><b>$$tasktitleref{$action}</b>
                  <br /><br />
                   $dateshow
                  </td>
                 </tr>
                </table>
ENDONE
    } elsif ($action eq 'updatenow') {
        $r->print("
                <br /><br />$dateshow\n");
    }
    $r->print(<<ENDTWO);
                <table width="100%" border="0" cellpadding="3" cellspacing="3">
                 <tr>
                  <td align="left" colspan="2">
                   <table border="0" cellspacing="0" cellpadding="2">
                    <tr>
                     <td colspan="3">
ENDTWO
      if ($action eq 'setaccess') {
          $r->print("<i>$lt{'setf'} $lt{'freg'}</i>");
      } elsif ($action eq 'updatenow') {
          $r->print("<i>$lt{'setf'} $lt{'fnew'}</i>");
      }
      $r->print(<<ENDTHREE); 
                       </td>
                      </tr>
                      <tr>
                       <td>$start_table
                       </td>
                      </tr>
                     </table>
                    </td>
                   </tr>
                   <tr>
                    <td align="left" colspan="2">
                     <table border="0" cellspacing="0" cellpadding="2">
                      <tr>
                       <td colspan="3">
ENDTHREE
    if ($action eq 'setaccess') {
        $r->print("<i>$lt{'setl'} $lt{'freg'}</i>");
    } elsif ($action eq 'updatenow') {
        $r->print("<i>$lt{'setl'} $lt{'fnew'}</i>");
    }
    $r->print(<<ENDFOUR); 
                       </td>
                      </tr>
                      <tr>
                       <td>$end_table
                       </td>
                      </tr>
                     </table>
                    </td>
                   </tr>
ENDFOUR
    if ($action eq 'setaccess') {
        $r->print("
                   <tr>
                    <td colspan=\"2\"><font color=\"#888888\">$lt{'ifad'}</font></td>
                   </tr>
                   <tr>
                    <td colspan=\"2\">&nbsp;</td>
                   </tr>
                   <tr>
                    <td colspan=\"2\"><b>".&mt('Note').":</b> ".$lt{'ncds'}."</td>
                   </tr>
                  </table>
        ");
    } elsif ($action eq 'updatenow') {
        $r->print("
                   </tr>
                  </table>
        ");
    }
}

###############################################################
sub print_doc_base {
    my $r = shift;
    $r->print(<<ENDBASE);
  </td>
 </tr>
</table>
<br/>
</body>
</html> 
ENDBASE
}
 
###################################################################
sub print_chgsettings_response {
    my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
    my %settings = &Apache::lonnet::get('environment',['internal.sectionnums','internal.crosslistings','internal.autostart','internal.autoend','internal.autoadds','internal.autodrops'],$dom,$crs);
    my $currend = '';
    my $currstart = '';
    my $currsecs = '';
    my $currxlists = '';
    my $curradds = '';
    my $currdrops = '';
    if ( defined($settings{'internal.autoadds'}) ) {
	$curradds = $settings{'internal.autoadds'};
    }
    if ( defined($settings{'internal.autodrops'}) ) {
	$currdrops = $settings{'internal.autodrops'};
    }
    if ( defined($settings{'internal.autostart'}) ) {
	$currstart = $settings{'internal.autostart'};
    }
    if ( defined($settings{'internal.autoend'}) ) {
	$currend = $settings{'internal.autoend'};
    }
    if ( defined($settings{'internal.sectionnums'}) ) {
	$currsecs = $settings{'internal.sectionnums'};
    }
    if ( defined($settings{'internal.crosslistings'}) ) {
	$currxlists = $settings{'internal.crosslistings'}
    }
    my $autoadds = '';
    my $autodrops = '';

    if ( exists($ENV{'form.autoadds'}) ) {
	$autoadds=$ENV{'form.autoadds'};
    }
    if ( exists($ENV{'form.autodrops'}) ) {
	$autodrops=$ENV{'form.autodrops'};
    }

    my $response = "";
    my $warning = "";
    my $warn_prefix = "";
    my $warn_suffix = "";
    my $warnfiller = "";
    my %cenv = ('internal.autoadds' => $autoadds,
		'internal.autodrops' => $autodrops);
    my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
    if ($reply !~ /^ok$/) {
	$response = "There was a problem processing your requested changes. The automated enrollment settings for this course have been left unchanged.<br/>";
    } else {
	if ($autoadds) {
	    if ($curradds) {
		$response = "Nightly additions based on classlist changes still <b>enabled</b><br/>";
	    } else {
		$response = "Nightly additions based on classlist changes now <b>enabled</b><br/>";
	    }
	} else {
	    if ($curradds) {
		$response = "Nightly additions based on classlist changes now <b>disabled</b><br/>";
	    } else {
		$response = "Nightly additions based on classlist changes still <b>disabled</b><br/>";
	    }
	}
	if ($autodrops) {
	    if ($currdrops) {
		$response .= "Nightly removals based on classlist changes still <b>enabled</b><br>";
	    } else {
		$response .= "Nightly removals based on classlist changes now <b>enabled</b><br/>";
	    }
	} else {
	    if ($currdrops) {
		$response .= "Nightly removals based on classlist changes now <b>disabled</b><br>";
	    } else {
		$response .= "Nightly removals based on classlist changes still <b>disabled</b>";
	    }
	}
	if ($autoadds || $autodrops) {
	    $warning = &warning_message($dom,$crs,$action);
	    $warn_prefix = "<br/><font size ='+1'><b>Warning</b></font>. Although you indicated that nightly ";
	    $warn_suffix = " should be enabled, additional action is required.<br/>";
	}
	if ($autoadds) {
	    if ($autodrops) {
		$warnfiller = "adds and drops";
	    } else {
		$warnfiller = "adds";
	    }
	} else {
	    if ($autodrops) {
		$warnfiller = "drops";
	    }
	}
	unless ($warning eq '') {
	    $response = $warn_prefix.$warnfiller.$warn_suffix.$warning;
	}
    }
    &print_reply($r,$response,$$tasktitleref{$action});
    return;
}

sub print_setdates_response {
    my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
    my %settings = &Apache::lonnet::get('environment',['internal.autostart','internal.autoend'],$dom,$crs);
    my $currstart = $settings{'internal.autostart'};
    my $currend = $settings{'internal.autoend'};
    my $response = '';
    my $showstart = '';
    my $showend = '';
    my $warning = '';
    my $warn_prefix = '';
    my ($autostart,$autoend) = &get_dates_from_form();
    if ( ($autostart eq '') || ($autoend eq '') ) {
        $response = "There was a problem processing your requested changes. The automated enrollment settings for this course have been left unchanged.<br/>";
    } elsif (($autoend > 0) && ($autoend <= $autostart)) {
        $response = 'The date/time selected for starting auto-enrollment was the same or later than the date/time selected for ending auto-enrollment. As this means auto-enrollment will never be active, your requested changes have not been processed, and the existing values remain in effect. Please <a href="javascript:history.go(-1)">go back</a> to the previous page to try your changes again.'."\n";
    } else {       
        $showstart = &Apache::lonlocal::locallocaltime($autostart);
        if ($autoend) {
	    $showend = &Apache::lonlocal::locallocaltime($autoend);
        } else {
	    $showend = "'No end date'";
        } 

        my %cenv = ('internal.autostart' => $autostart,
  		    'internal.autoend' => $autoend);
        my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
        if ($reply !~ /^ok$/) {
	    $response = "There was a problem processing your requested changes. The automated enrollment settings for this course have been left unchanged.<br/>";
        } else {
	    if ($currstart == $autostart) {
	        $response = "The first date for automated enrollment has been left unchanged as $showstart.<br/>";
	    } else {
	        $response = "The first date for automated enrollment has been changed to $showstart.<br/>";
	    } 
	    if ($currend == $autoend) {
	        $response .= "The last date for automated enrollment has been left unchanged as $showend.<br/>";
	    } else {
	        $response .= "The last date for automated enrollment has been changed to $showend.<br/>";
	    }
 
# Generate message in case where old first enrollment date was later than today, but new first enrollment date is now today or earlier.

	    my $rosterupdated = 0;
	    my $firstaccess = "";
	    my $nextupdate = "";
	    my $lastupdate = "";

	    my $nowstamp = time;
	    my @date_list=localtime(time);
	    my $cur_year = $date_list[5];
	    my $curday = $date_list[3];
	    my $curmonth = $date_list[4];
	    my $lastmidnt = timelocal(0,0,0,$date_list[3],$date_list[4],$date_list[5]);
	    my $nextmidnt = 86400 + $lastmidnt;

	    my $todayupdate = timelocal(0,30,1,$date_list[3],$date_list[4],$date_list[5]);
	    my $lastupdate = $todayupdate - 86400;
	    if ($nowstamp < $todayupdate) {
	      $nextupdate = "today";
	    } else {
	      $nextupdate = "tomorrow";
	    }
	    if ($currstart < $lastupdate) {
	        $rosterupdated = 1;
	    }
	    if ($autostart < $nextmidnt ) {
	        if ( $autostart >= $lastmidnt) {
		    $firstaccess = "today";
	        } else {
	  	    $firstaccess = "a date prior to today";
	        }
	        if (($nowstamp >= $autostart) && ($rosterupdated == 0)) {
		    $response .= qq|<br>Although you have now set the first enrollment date to $firstaccess, automatic enrollment will <b>not</b> occur until the next automatic enrollment update occurs for all LON-CAPA courses at 1.30 am $nextupdate. If you wish to immediately enroll registered students included in the institutional classlist for this class, please visit the <a href="/adm/populate?action=updatenow">roster update page</a>.<br>|;
	        }
	    }
	    $warning = &warning_message($dom,$crs,$action);
	    $warn_prefix = "<br/><font size ='+1'><b>Warning</b></font>. Although you set a start and end date for auto-enrollment, additional action is required.<br/>";
	    unless ($warning eq '') {
	        $response .= $warn_prefix.$warning;
	    }
        }
    }
    &print_reply($r,$response,$$tasktitleref{$action});
    return;
}

sub print_setaccess_response {
    my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
    my %settings = &Apache::lonnet::get('environment',['default_enrollment_start_date','default_enrollment_end_date','internal.autostart'],$dom,$crs);
    my $currstart = $settings{'default_enrollment_start_date'};
    my $currend = $settings{'default_enrollment_end_date'};
    my $autostart = $settings{'internal.autostart'};
    my $response = '';
    my ($startaccess,$endaccess) = &get_dates_from_form();
    if (($startaccess eq '') || ($endaccess eq '')) {
        $response = "There was a problem processing your requested changes. The default
start and end access dates for this course have been left unchanged.<br/>";
    } elsif (($endaccess > 0) && ($endaccess <= $startaccess)) {
        $response = 'The default start access date/time you chose was the same or later than the default end access date/time. As this means that roles will never be active, your requested changes have not been processed, and the existing values remain in effect. Please <a href="javascript:history.go(-1)">go back</a> to the previous page to try your changes again.'."\n"; 
    } else {
        my $showstart = &Apache::lonlocal::locallocaltime($startaccess);
        my $showend = '';
        my $warning = '';
        my $warn_prefix = '';
        if ($endaccess) {
	    $showend = &Apache::lonlocal::locallocaltime($endaccess);
        } else {
	    $showend = "'No end date'";
        }

        my %cenv = ('default_enrollment_start_date' => $startaccess,
		'default_enrollment_end_date' => $endaccess);
        my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
        if ($reply !~ /^ok$/) {
	    $response = "There was a problem processing your requested changes. The automated enrollment settings for this course have been left unchanged.<br/>";
        } else {
	    if ($currstart == $startaccess) {
	        $response = "The first access date for students added via automated enrollment has been left unchanged as $showstart.<br/>";
	    } else {
	        $response = "The first access date for students added via automated enrollment has been changed to
$showstart.<br/>";
	    }
	    if ($currend == $endaccess) {
	        $response .= "The last access date for students added via automated enrollment has been left unchanged as $showend.<br/>";
	    } else {
	        $response .= "The last access date for students automated enrollment has been changed to
$showend.<br/>";
	    }
	
# Generate message in case where old first access date was later than today, but new first access date is now today or earlier.
	
	    my $accessgiven= 0;
	    my $firstaccess = "";
	    my $nextupdate = "";
	    my $lastupdate = "";

	    my $nowstamp = time;
	    my @date_list=localtime(time);
	    my $cur_year = $date_list[5];
	    my $curday = $date_list[3];
	    my $curmonth = $date_list[4];
	    my $lastmidnt = timelocal(0,0,0,$date_list[3],$date_list[4],$date_list[5]);
	    my $nextmidnt = 86400 + $lastmidnt;

	    my $todayupdate = timelocal(0,30,1,$date_list[3],$date_list[4],$date_list[5]);
	    my $tomorrowupdate = $todayupdate + 86400;
	    my $lastupdate = $todayupdate - 86400;

	    if ($autostart < $nextmidnt) {
	        if ($nowstamp < $todayupdate) {
		    $nextupdate = "at 1.30 am today";
	        } else {
		    $nextupdate = "at 1.30 am tomorrow";
	        }
	    } else {
	        my @enrollstart = localtime($autostart);
	        $nextupdate = timelocal(0,30,1,$enrollstart[3],$enrollstart[4],$enrollstart[5]);
	        unless (($enrollstart[2] < 1) || ($enrollstart[2] == 1 && $enrollstart[1] <=30))  {
		    $nextupdate += 86400; 
	        }
	        $nextupdate = &Apache::lonlocal::locallocaltime($nextupdate);
	    }
	    if (($currstart < $lastupdate) && ($autostart < $lastupdate)) {
	        $accessgiven = 1;
	    }
	    if ($startaccess < $nextmidnt ) {
	        if ( $startaccess >= $lastmidnt) {
	    	    $firstaccess = "today";
	        } else {
		    $firstaccess = "a date prior to today";
	        }
	        if (($nowstamp >= $startaccess) && ($accessgiven == 0)) {
		    $response .= qq|<br>Although you have now set the first access date to $firstaccess, automatic enrollment will <b>not</b> occur until the next automatic enrollment update occurs for all LON-CAPA courses $nextupdate. If you wish to grant immediate course access for registered students included in the institutional classlist for this class, please visit the <a href="/adm/populate?action=updatenow">roster update page</a>.<br>|;
	        }
	    }
	    $warning = &warning_message($dom,$crs,$action);
	    $warn_prefix = "<br/><font size ='+1'><b>Warning</b></font>. Although you have set default first and last access dates for students who are added via automatic enrollment, additional action is required.<br/>";
	    unless ($warning eq '') {
	        $response .= $warn_prefix.$warning;
	    }
        }
    }
    &print_reply($r,$response,$$tasktitleref{$action});
    return;
}

sub print_notify_response {
    my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;

# Get current settings
    my %settings = &Apache::lonnet::get('environment',['internal.notifylist','internal.coursecode'],$dom,$crs);
    my $notifylist = $settings{'internal.notifylist'};
    my $coursecode = $settings{'internal.coursecode'};
    my @currpeople = ();
    my $notify = 0;
    my @people = ();
    my $peoplestr = "";
    my $response = "";
    my $noprocess = 0;

    if ($notifylist =~ m/,/) {
	@currpeople = split/,/,$notifylist;
    } else {
	$currpeople[0] = $notifylist;
    }
    my $currcount = 0;
    foreach  (@currpeople) {
	unless ($_ eq '') { $currcount ++; } 
    }

    if ( exists($ENV{'form.notify'}) ) {
	$notify=$ENV{'form.notify'};
    }
    if ( exists($ENV{'form.notifyshow'}) ) {
	my $notifyshow = $ENV{'form.notifyshow'};
	for (my $i=0; $i<$notifyshow; $i++) {
	    if ( exists($ENV{"form.note_$i"}) ) {
		if ( exists($ENV{"form.notifyname_$i"}) ) {
		    unless ( $ENV{"form.notifyname_$i"} eq '' ) {
			push @people, $ENV{"form.notifyname_$i"};
		    }
		}
	    }
	}
	if ($notify) { $peoplestr = join(",",@people); }
    } else {
	if ($notify) {
	    if ($currcount) {
		$response = "There was a problem retrieving the updated list of recipients of notification messages. The notification settings for this course have been left unchanged.<br/>.";
		$peoplestr = $notifylist;
		@people = @currpeople;
		$noprocess = 1;
	    }
	}
    }
    unless ($noprocess == 1) {
	my %cenv = ('internal.notifylist' => $peoplestr);
	my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
	if ($reply !~ /^ok$/) {
	    $response = "There was a problem processing your requested changes. The notification settings for this course have been left unchanged.<br/>";
	} else {
	    if ($notify) {
		if (@people) {
		    if ($currcount) {
			$response .= "Notification of enrollment changes still <b>enabled</b><br/>";
		    } else {
			$response .= "Notification of enrollment changes now <b>enabled</b><br/>";
		    }
		    $response .= "<br/>The following will receive notification if there are any changes in enrollment in LON-CAPA course: $realm ($coursecode) as a result of the nightly enrollment check: <br/><ul>\n";
		    foreach my $person (@people) {
			$response .= "<li>$person</li>\n";
		    }
		    $response .= "</ul>\n";
		} else {
		    $response = "Notification of enrollment changes was <b> not enabled</b> as no course coordinators were selected as recipients.<br/>";
		}
	    } else {
		if ($currcount) {
		    $response = "Notification of enrollment changes now <b>disabled</b><br/>";
		} else {
		    $response = "Notification of enrollment changes still <b>disabled</b><br/>";
		}
	    }
	}
    }
    &print_reply($r,$response,$$tasktitleref{$action});
    return;
}

sub print_crosslistings_menu () {
    my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
    my %settings = &Apache::lonnet::get('environment',['internal.crosslistings','internal.coursecode'],$dom,$crs);
    my @currxlists = ();
    my @xlists = ();
    my $crosscount = 0;
    my $removecount = 0;
    my $xliststr = '';
    my $response = '';
    my $coursecode = $settings{'internal.coursecode'};
    if ($settings{'internal.crosslistings'} =~ m/,/) {
	@currxlists = split/,/,$settings{'internal.crosslistings'};
    } elsif ($settings{'internal.crosslistings'} ne '') {
	$currxlists[0] = $settings{'internal.crosslistings'};
    }

    if (@currxlists > 0) {
	for (my $i=0; $i<@currxlists; $i++) {
	    my $xlist = "cross_".$i;
	    my $gp = "gp_".$i;
	    if ( exists($ENV{"form.$xlist"}) ) {
		my $xlistentry = '';
		if ($currxlists[$i] =~ m/^([^:]+)/) {
		    $xlistentry = $1.':';
		}
		if ( exists($ENV{"form.$gp"}) ) {
		    $xlistentry .= $ENV{"form.$gp"};
		}
		push @xlists,$xlistentry;
		$crosscount ++;
	    } else {
		$removecount ++;
	    }
	}
    }

    if ($crosscount > 1) {
	$xliststr = join(",",@xlists);
    } else {
	$xliststr = $xlists[0];
    }
    my %cenv = ('internal.crosslistings' => $xliststr);
    my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
    if ($reply !~ /^ok$/) {
	$response = "There was a problem processing your requested changes. The automated enrollment settings for this course have been left unchanged.<br/>";
    } else {
	if ($removecount > 0) {
	    $response = "A total of $removecount courses are no longer crosslisted with LON-CAPA course: $realm ($coursecode).<br/><br/>";
	}
	if ($crosscount > 0) {
	    $response .=  "The $crosscount courses listed below remain crosslisted with this LON-CAPA course, and students enrolling in these course sections will be automatically added to the class roster for the course, if you have chosen to enable a nightly automated enrollment update.<br/><ul>\n";
	    foreach (@xlists) {
		my ($xlist,$gp) = split/:/,$_;
		$response .= "<li>$xlist - ID: $gp</li>\n";
	    }
	    $response .= "</ul><br/>\n";
	}
    }
    if ( exists($ENV{'form.numcross'}) ) {
	my $numcross = $ENV{'form.numcross'};
	if ($numcross > 0) {
	    my @bgcolors=("#eeeeee","#cccccc");
	    $response .= qq(You indicated that you wish to add an additional $numcross crosslisting(s).  For each new crosslisting enter the insititutional course section code (e.g., fs03zol101001, for section 001 of zol101 for fs03 semester), and the LON-CAPA section/group ID you wish to assign to students who will be enrolled in your LON-CAPA class as a result of their registration in the crosslisted course section. The LON-CAPA section/group ID can be left blank, if you do not wish to tie a section/group ID to this crosslisting.  The institutional course section code should only contain letters and/or numbers, and must be consistent with the scheme adopted by your Domain Coordinator to map course codes (and section numbers) to your institution\'s student information system.<br/><br/>
           <form name="enter" method="post">
           <table border="0" cellpadding="2" cellspacing="2" width="100%">
             <tr>
              <td>
               <table border="0" cellspacing="3" cellpadding="3">
                <tr bgcolor="#CCCCFF" align="left">
                 <td><b>Crosslisting</b></td>
                 <td><b>LON-CAPA section/groupID</b></td>
                </tr>
			    );
	    for (my $i=0; $i<$numcross; $i++) {
		my $colflag = $i%2;
		$response .= qq(
                <tr bgcolor="$bgcolors[$colflag]" align="left">
                 <td><input type="text" size="15" name="newcross_$i" /></td>
                 <td><input type="text" size="10" name="newgroupid_$i" /></td>
                </tr>
				);
	    }
	    $response .= qq(  </table>
              </td>
             </tr>
             <tr>
              <td align="right">
               <input type="button" name="newcross" value="Go" onClick="process('newcross')" />
              </td>
             </tr>
            </table>
            <input type=\"hidden\" name=\"numcross\" value=\"$numcross\">
            <input type=\"hidden\" name=\"action\" value=\"newcross\">
            <input type=\"hidden\" name=\"state\" value=\"process\">
            </form>
			      );
        }
    }
    &print_reply($r,$response,$$tasktitleref{$action});
    return;
}

sub print_crosslistings_response () {
    my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
    my %settings = &Apache::lonnet::get('environment',['internal.crosslistings','internal.coursecode','internal.courseowner'],$dom,$crs);
    my @currxlists = ();
    my @xlists = ();
    my @allxlists = ();
    my @badxlists = ();
    my @badowner = ();
    my $numcross = 0;
    my $xliststr =  $settings{'internal.crosslistings'};
    my $coursecode = $settings{'internal.coursecode'};
    my $owner = $settings{'internal.courseowner'};
    my $response = '';
    my $warning = '';
    my $warn_prefix = '';

    if ($xliststr =~ m/,/) {
	@allxlists = split/,/,$xliststr;
    } else {
	unless ($xliststr eq '') { $allxlists[0] = $xliststr; }
    }

    if ( exists($ENV{'form.numcross'}) ) {
	$numcross = $ENV{'form.numcross'};
    }

    if ($numcross > 0) {
	for (my $i=0; $i<$numcross; $i++) {
	    my $xl = "newcross_".$i;
	    my $gp = "newgroupid_".$i;
	    if ( exists($ENV{"form.$xl"}) ) {
		my $coursecheck = '';
		$coursecheck = &Apache::lonnet::auto_validate_courseID($crs,$dom,$ENV{"form.$xl"});
		if ($coursecheck eq 'ok') {
		    my $addcheck = '';
		    $addcheck = &Apache::lonnet::auto_new_course($crs,$dom,$ENV{"form.$xl"},$owner);
		    if ($addcheck eq 'ok') {
			push @xlists,$ENV{"form.$xl"}.":".$ENV{"form.$gp"};
		    } else {
			push @badowner,$ENV{"form.$xl"}.":".$ENV{"form.$gp"};
		    }
		} else {
		    push @badxlists, $ENV{"form.$xl"}.":".$ENV{"form.$gp"}.":".$coursecheck;
		}
	    }
	}
	push @allxlists, @xlists;
    }
    
    if (@xlists > 0 ) {
	unless ($xliststr eq '') { $xliststr .= ","; }
	if (@xlists > 1) {
	    $xliststr .= join(",",@xlists);
	} else {
	    $xliststr .= $xlists[0];
	}
	my %cenv = ('internal.crosslistings' => $xliststr);
	my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
	if ($reply !~ /^ok$/) {
	    $response = "There was a problem processing your requested changes. The automated enrollment settings for this course have been left unchanged.<br/><br/>";
	} else {
	    $response = "The courses listed below are now crosslisted with this LON-CAPA course, and students enrolling in these course sections will be automatically added to the class roster for the course, if you have chosen to enable a nightly automated enrollment update.<br/><ul>\n";
	    foreach (@allxlists) {
		my ($xlist,$gp) = split/:/,$_;
		$response .= "<li>$xlist - ID: $gp</li>\n";
	    }
	    $response .= "</ul><br/><br/>\n";
	}
    } else {
	if ($xliststr =~ m/:/) {
	    my @oldxlists = ();
	    if ($xliststr =~ m/,/) {
		@oldxlists = split/,/,$xliststr;
	    } else {
		$oldxlists[0] = $xliststr;
	    }
	    $response .= "Although no new crosslistings were added, the courses listed below continue to be crosslisted with your LON-CAPA course.<br/><ul>\n";
	    foreach (@oldxlists) {
		my ($xlist,$gp) = split/:/,$_;
		$response .= "<li>$xlist - ID: $gp</li>\n";
	    }
	    $response .= "</ul><br/><br/>\n";
	}
    }
    if (@badxlists > 0) {
	$response .= "The courses listed below could not be included in the crosslistings for this LON-CAPA course, because they are not valid courses according to your institution's official schedule of classes and sections.<br/><ul>\n";
	foreach (@badxlists) {
	    my ($xlist,$gp,$prob) = split/:/,$_;
	    $response .= "<li>$xlist - ID: $gp - Error: $prob</li>\n";
	}
	$response .= "</ul><br/><br/>\n";
    }
    
    if (@badowner > 0) {
	$response .= "The courses listed below could not be included in the crosslistings for this LON-CAPA course, because the owner of this course - $owner - does not have rights to view enrollment in those classes as determined by your instititution's policies on access to official classlists.<br/><ul>\n";
	foreach (@badowner) {
	    my ($xlist,$gp) = split/:/,$_;
	    $response .= "<li>$xlist - ID: $gp</li>\n";
	}
	$response .= "</ul><br/><br/>\n";
    }

    if (@allxlists > 0) {
	$warning = &warning_message($dom,$crs,$action);
	$warn_prefix = "<br/><font size ='+1'><b>Warning</b></font>. Although you have selected crosslisted courses to contribute enrollment to this course, additional action is required.<br/>";
	unless ($warning eq '') {
	    $response .= $warn_prefix.$warning;
	}
    }

    &print_reply($r,$response,$$tasktitleref{$action});
    return;
}

sub print_sections_menu () {
    my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
    my %settings = &Apache::lonnet::get('environment',['internal.sectionnums','internal.coursecode','internal.courseowner'],$dom,$crs);
    my @currsections = ();
    my @sections = ();
    my @badowner = ();
    my @badsections = ();
    my $seccount = 0;
    my $removecount = 0;
    my $addcount = 0;
    my $secstr = '';
    my $response = '';
    my $warning = "";
    my $warn_prefix = "";
    my $coursecode = $settings{'internal.coursecode'};
    my $owner = $settings{'internal.courseowner'};
    if ($settings{'internal.sectionnums'} =~ m/,/) {
	@currsections = split/,/,$settings{'internal.sectionnums'};
    } elsif ($settings{'internal.sectionnums'} ne '') {
	$currsections[0] = $settings{'internal.sectionnums'};
    }
    
    if ( exists($ENV{'form.secshow'}) ) {
	for (my $i=0; $i<$ENV{'form.secshow'}; $i++) {
	    my $gp = "loncapasec_".$i;
	    my $secnum = "secnum_".$i;
	    my $sec = "sec_".$i;
	    if ( exists( $ENV{"form.$sec"} ) ) {
		my $secentry;
		if ( exists( $ENV{"form.$secnum"} ) ) { 
		    $secentry = $ENV{"form.$secnum"}.':';
		}
		if ( exists( $ENV{"form.$gp"} ) ) {
		    $secentry .= $ENV{"form.$gp"};
		}
		if ( grep/^$ENV{"form.$secnum"}:/,@currsections) {
		    push @sections, $secentry;
		    $seccount ++;
		} else {
		    my $newsec = $coursecode.$ENV{"form.$secnum"};
		    my $coursecheck = &Apache::lonnet::auto_validate_courseID($crs,$dom,$newsec);
		    if ($coursecheck eq 'ok') {
			my $addcheck = &Apache::lonnet::auto_new_course($crs,$dom,$newsec,$owner);
			if ($addcheck eq 'ok') {
			    push @sections,$ENV{"form.$secnum"}.":".$ENV{"form.$gp"};
			    $seccount ++;
			    $addcount ++;
			} else {
			    push @badowner,$ENV{"form.$secnum"}.":".$ENV{"form.$gp"};
			}
		    } else {
			push @badsections, $ENV{"form.$secnum"}.":".$ENV{"form.$gp"}.":".$coursecheck;
		    }
		}
	    }
	}
	if (@currsections > 0) {
	    for (my $i=0; $i<@currsections; $i++) {
		if ($currsections[$i] =~ m/^(\w+:)/ ) {
		    my $oldsec  = $1;
		    unless (grep/^$oldsec/,@sections) {
			$removecount ++;
		    }
		}
	    }
	}
    } elsif (@currsections > 0) {
	for (my $i=0; $i<@currsections; $i++) {
	    my $sec = "sec_".$i;
	    my $gp = "secgp_".$i;
	    if ( exists($ENV{"form.$sec"}) ) {
		my $secentry = '';
		if ($currsections[$i] =~ m/^(\w+:)/ ) {
		    $secentry = $1;
		}
		if ( exists($ENV{"form.$gp"}) ) {
		    $secentry .= $ENV{"form.$gp"};
		}
		push @sections,$secentry;
		$seccount ++;
	    } else {
		$removecount ++;
	    }
	}
    }
    
    if ($seccount > 1) {
	$secstr = join(",",@sections);
    } else {
	$secstr = $sections[0];
    }
    my %cenv = ('internal.sectionnums' => $secstr);
    my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
    if ($reply !~ /^ok$/) {
	$response = "There was a problem processing your requested changes. The automated enrollment settings for this course have been left unchanged.<br/>";
    } else {
	if ($removecount > 0) {
	    $response = "A total of $removecount sections have been removed from the list of sections which contribute to enrollment in LON-CAPA course: $realm ($coursecode).<br/><br/>";
	}
	if ($addcount > 0) {
	    $response .= "A total of $addcount sections have been added to the list of
sections which contribute to enrollment in LON-CAPA course: $realm ($coursecode).<br/><br/>";
	}
	if ($seccount > 0) { 
	    $response .= "Students enrolling in the $seccount section(s) listed below will be automatically added to the class roster for the course, if you have chosen to enable a nightly automated enrollment update.<br/><ul>\n";
	    foreach (@sections) {
		my ($sec,$gp) = split/:/,$_;
		$response .= "<li>$sec  - ID: $gp</li>\n";
	    }
	    $response .= "</ul><br/>\n";
	}
    }
    
    if (@badsections > 0) {
	$response .= "The sections listed below could not be included in the sections for this LON-CAPA course, because they are not valid section numbers according to your institution's official schedule of classes and sections.<br/><ul>\n";
	foreach (@badsections) {
	    my ($secnum,$gp,$prob) = split/:/,$_;
	    $response .= "<li>$secnum - ID: $gp - Error: $prob</li>\n";
	}
	$response .= "</ul><br/><br/>\n";
    }
    
    if (@badowner > 0) {
	$response .= "The sections listed below could not be included in the sections for this LON-CAPA course, because the owner of this course - $owner - does not have rights to view enrollment in those classes as determined by your instititution's policies on access to official classlists.<br/><ul>\n";
	foreach (@badowner) {
	    my ($secnum,$gp) = split/:/,$_;
	    $response .= "<li>$secnum - ID: $gp</li>\n";
	}
	$response .= "</ul><br/><br/>\n";
    }
    
    if ($seccount > 0) {
	$warning = &warning_message($dom,$crs,$action);
	$warn_prefix = "<br/><font size ='+1'><b>Warning</b></font>. Although you have selected sections to contribute enrollment to this course, additional action is required.<br/>";
	unless ($warning eq '') {
	    $response .= $warn_prefix.$warning;
	}
    }
    
    if ( exists($ENV{'form.numsec'}) ) {
	my $numsec = $ENV{'form.numsec'};
	if ($numsec > 0) {
	    my @bgcolors=("#eeeeee","#cccccc");
	    $response .= qq(
			    You indicated that you wish to incorporate student enrollment in your LON-CAPA course from an additional $numsec section(s).  For each new section enter the insititutional section code (e.g., 004), and the LON-CAPA section/group ID you wish to assign to students who will be enrolled in your LON-CAPA class as a result of their registration in this particular section. The LON-CAPA section/group ID can be left blank, if you do not wish to designate a section/group ID for this course section.  The institutional section code should only contain letters and/or numbers, and must be consistent with the scheme adopted by your Domain Coordinator to map course section numbers to your institution\'s student information system.<br/><br/> 
           <form name="enter" method="post">
           <table border="0" cellpadding="2" cellspacing="2" width="100%">
             <tr>
              <td>
               <table border="0" cellspacing="3" cellpadding="3">
                <tr bgcolor="#CCCCFF" align="left">
                 <td><b>Section number</b></td>
                 <td><b>LON-CAPA section/groupID</b></td>
                </tr>
			    );
	    for (my $i=0; $i<$numsec; $i++) {
		my $colflag = $i%2;
		$response .= qq(
                <tr bgcolor="$bgcolors[$colflag]" align="left">
                 <td><input type="text" size="15" name="newsec_$i" /></td>
                 <td><input type="text" size="10" name="newsecgp_$i" /></td>
                </tr>
				);
	    }
	    $response .= qq(  </table>
              </td>
             </tr>
             <tr>
              <td align="right">
               <input type="button" name="newsections" value="Go" onClick="process('newsections')" />
              </td>
             </tr>
            </table>
            <input type=\"hidden\" name=\"numsec\" value=\"$numsec\">
            <input type=\"hidden\" name=\"action\" value=\"newsections\">
            <input type=\"hidden\" name=\"state\" value=\"process\">
            </form>
			      );
	}
    }
    &print_reply($r,$response,$$tasktitleref{$action});
    return;
}

sub print_sections_response () {
    my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
    my %settings = &Apache::lonnet::get('environment',['internal.sectionnums','internal.coursecode','internal.courseowner'],$dom,$crs);
    my @currsections = ();
    my @sections = ();
    my @allsections = ();
    my @badowner = ();
    my @badsections = ();
    my $numsec = 0;
    my $secstr =  $settings{'internal.sectionnums'};
    my $coursecode = $settings{'internal.coursecode'};
    my $owner = $settings{'internal.courseowner'};
    my $response = '';
    my $putreply = '';
    my $warning = '';
    my $warn_prefix = '';
    if ($secstr =~ m/,/) {
	@allsections = split/,/,$secstr;
    } else {
	unless ($secstr eq '') { $allsections[0] = $secstr; }
    }
    
    if ( exists($ENV{'form.numsec'}) ) {
	$numsec = $ENV{'form.numsec'};
    }
    
    if ($numsec > 0) {
	for (my $i=0; $i<$numsec; $i++) {
	    my $sec = "newsec_".$i;
	    my $gp = "newsecgp_".$i;
	    if ( exists($ENV{"form.$sec"}) ) {
		unless ( (grep/^$ENV{"form.$sec"}:/,@allsections) || (grep/^$ENV{"form.$sec"}:/,@sections) ) {
		    my $newsec = $coursecode.$ENV{"form.$sec"};
		    my $coursecheck = &Apache::lonnet::auto_validate_courseID($crs,$dom,$newsec);
		    if ($coursecheck eq 'ok') {
			my $addcheck = &Apache::lonnet::auto_new_course($crs,$dom,$newsec,$owner);
			if ($addcheck eq 'ok') {
			    push @sections,$ENV{"form.$sec"}.":".$ENV{"form.$gp"};
			} else {
			    push @badowner,$ENV{"form.$sec"}.":".$ENV{"form.$gp"};
			}
		    } else {
			push @badsections, $ENV{"form.$sec"}.":".$ENV{"form.$gp"}.":".$coursecheck;
		    }
		}
	    }
	}
	push @allsections, @sections;
    }
    
    if (@sections > 0 ) {
	unless ($secstr eq '') { $secstr .= ","; } 
	if (@sections > 1) {
	    $secstr .= join(",",@sections);
	} else {
	    $secstr .= $sections[0];
	}
	my %cenv = ('internal.sectionnums' => $secstr);
	$putreply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
	if ($putreply !~ /^ok$/) {
	    $response = "There was a problem processing your requested changes. The automated enrollment settings for this course have been left unchanged.<br/><br/>";
	}
    }

    if ($putreply =~ /^ok/) {
	$response = "Students enrolling in the sections listed below will be automatically added to the class roster for LON-CAPA course $realm ($coursecode), if you have chosen to enable a nightly automated enrollment update.<br/><ul>\n";
	foreach (@allsections) {
	    my ($sec,$gp) = split/:/,$_;
	    $response .= "<li>$sec - ID: $gp</li>\n";
	}
	$response .= "</ul><br/><br/>\n";
    }

    if (@badsections > 0) {
	$response .= "The sections listed below could not be included in the sections for this LON-CAPA course, because they are not valid section numbers according to your institution's official schedule of classes and sections.<br/><ul>\n";
	foreach (@badsections) {
	    my ($secnum,$gp,$prob) = split/:/,$_;
	    $response .= "<li>$secnum - ID: $gp - Error: $prob</li>\n";
	}
	$response .= "</ul><br/><br/>\n";
    }

    if (@badowner > 0) {
	$response .= "The sections listed below could not be included in the sections for this LON-CAPA course, because the owner of this course - $owner - does not have rights to view enrollment in those classes as determined by your instititution's policies on access to official classlists.<br/><ul>\n";
	foreach (@badowner) {
	    my ($secnum,$gp) = split/:/,$_;
	    $response .= "<li>$secnum - ID: $gp</li>\n";
	}
	$response .= "</ul><br/><br/>\n";
    }

    if (@allsections > 0) {
	$warning = &warning_message($dom,$crs,$action);
	$warn_prefix = "<br/><font size ='+1'><b>Warning</b></font>. Although you have selected sections to contribute enrollment to this course, additional action is required.<br/>";
	unless ($warning eq '') {
	    $response .= $warn_prefix.$warning;
	}
    }

    &print_reply($r,$response,$$tasktitleref{$action});
    return;
}

sub print_photos_response () {
    my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
    my %settings = &Apache::lonnet::get('environment',['internal.showphotos'],$dom,$crs);
    my $currphotos = $settings{'internal.showphotos'};
    my $showphotos = '';
    if ( exists($ENV{'form.showphotos'}) ) {
	$showphotos=$ENV{'form.showphotos'};
    }
    
    my $response = "";
    my %cenv = ('internal.showphotos' => $showphotos);
    my $reply = &Apache::lonnet::put('environment',\%cenv,$dom,$crs);
    if ($reply !~ /^ok$/) {
	$response = "There was a problem processing your requested change. The student photo import setting for this course has been left unchanged.<br/>";
    } else {
	if ($showphotos) {
	    if ($currphotos) {
		$response = "Retrieval of student photos is still <b>enabled</b><br/>";
	    } else {
		$response = "Retrieval of student photos in now <b>enabled</b><br/>";
	    }
	} else {
	    if ($currphotos) {
		$response = "Retrieval of student photos is now <b>disabled</b><br/>";
	    } else {
		$response = "Retrieval of student photos is still <b>disabled</b><br/>";
	    }
	}
    }
    &print_reply($r,$response,$$tasktitleref{$action});
    return;
}

sub print_update_result () {
    my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
    my $response = '';
    my $updateadds = 0;
    my $updatedrops = 0;
    my $changecount = 0;
    my %affiliates = ();
    my %reply = ();
    my @allcourses = ();
    my %LC_code = ();
    my $logmsg = '';
    my $newusermsg = '';

    my %settings = &Apache::lonnet::get('environment',['internal.coursecode','internal.sectionnums','internal.crosslistings','internal.authtype','internal.autharg'],$dom,$crs);
    my $coursecode = $settings{'internal.coursecode'};
    my $authtype = $settings{'internal.authtype'};
    my $autharg = $settings{'internal.autharg'};
    my ($startaccess,$endaccess) = &get_dates_from_form();
    if ( exists($ENV{'form.updateadds'}) ) {
        $updateadds = $ENV{'form.updateadds'};
    }
    if ( exists($ENV{'form.updatedrops'}) ) {
        $updatedrops = $ENV{'form.updatedrops'};
    }
    if (($startaccess eq '') || ($endaccess eq '')) {
        $response = "There was a problem processing your requested roster update because start and and access dates could not be determined. No changes have been made to the class roster.<br />"; 
    } elsif ($updateadds && (($endaccess > 0) && ($endaccess <= $startaccess))) {
        $response = 'The start access date/time is the same or later than the end access date/time. As this means that new roles will never be active, your requested roster update has not been carried out, and the roster remains unchanged. Please <a href="javascript:history.go(-1)">go back</a> to the previous page to try your changes again.'."\n";
    } elsif (!$updateadds && !$updatedrops) {
	$response = "An update of the class roster has not been carried out because you indicated that you wanted to neither add new students, nor expire dropped students based on a comparison between the institutional class lists for the course sections and crosslisted courses that contribure enrollment to this LON-CAPA course.";
    } elsif ($coursecode eq '') {
	$response = "There was a problem retrieving the course code for this LON-CAPA course.  An update of the class roster has not been carried out, and enrollment remains unchanged";
    } else {
# Get complete list of course sections to update
	my @currsections = ();
	my @currxlists = ();
	
	if ($settings{'internal.sectionnums'} =~ m/,/) {
	    @currsections = split/,/,$settings{'internal.sectionnums'};
	} elsif ($settings{'internal.sectionnums'} ne '') {
	    $currsections[0] = $settings{'internal.sectionnums'};
	}
	
	if ($settings{'internal.crosslistings'} =~ m/,/) {
	    @currxlists = split/,/,$settings{'internal.crosslistings'};
	} elsif ($settings{'internal.crosslistings'} ne '') {
	    $currxlists[0] = $settings{'internal.crosslistings'};
	}
	
	if (@currxlists > 0) {
	    foreach (@currxlists) {
		if (m/^(\w+):(\w*)$/) {
		    unless (grep/^$1$/,@allcourses) { 
			push @allcourses,$1;
			$LC_code{$1} = $2; 
		    }
		}
	    }
	}
	
	if (@currsections > 0) {
	    foreach (@currsections) {
		if (m/^(\w+):(\w*)$/) {
		    my $sec = $coursecode.$1;
		    my $gp = $2;
		    unless (grep/^$sec$/,@allcourses) { 
			push @allcourses,$sec;
			$LC_code{$sec} = $gp; 
		    }
		}
	    }
	}

	if (@allcourses > 0) {
	    @{$affiliates{$crs}} = @allcourses;
	    my $outcome = &Apache::lonnet::fetch_enrollment_query('updatenow',\%affiliates,\%reply,$dom,$crs);
            unless ($outcome eq 'ok') {
                &Apache::lonnet::logthis("lonpopulate:update roster".
                                         "failed to retrieve classlist".
                                 " data for ".$crs."\@".$dom." by ".
                                 $ENV{'user.name'}." \@ ".$ENV{'user.domain'}.
                                 ": ".$outcome);
            }
	    if ($reply{$crs} > 0) {
		($changecount,$response) = &LONCAPA::Enrollment::update_LC($dom,$crs,$updateadds,$updatedrops,$startaccess,$endaccess,$authtype,$autharg,\@allcourses,\%LC_code,\$logmsg,\$newusermsg,"updatenow");
	    } else {
		$response = "There was a problem retrieving institutional class list data for the course sections and crosslisted courses which contribute enrollment to this course. No updates have been carried out, and the roster remains unchanged.";
	    }  
	} else {
	    $response = "There are currently no course sections or crosslisted courses designated as contributors to enrollment in this LON-CAPA course. As a result a student roster update has not been carried out for $realm ($coursecode)";
	}
    }
    unless ($logmsg eq '') {
	my $loglength = length($logmsg);
	$logmsg = substr($logmsg,0,$loglength-4);
	$logmsg = "<br/><br/>The following messages were generated by the roster update process: <br/><ul><li>".$logmsg."</ul><br/>";
    }
    unless ($newusermsg eq '') {
	$newusermsg = substr( $newusermsg,0,rindex($newusermsg,'<li>') );
	$newusermsg = "<br/><br/>The following new system user(s) who was/were created will be using local or internal authentication with an initial randomly generated password. A valid e-mail address was not available for this/these user(s) so LON-CAPA account credentials could not be sent via e-mail.<br/><ul><li>".$newusermsg."</ul><br/>";
    }
    $response .= $logmsg.$newusermsg;
    &print_reply($r,$response,$$tasktitleref{$action});
    return;
}

sub print_viewclass_response {
    my ($r,$realm,$dom,$crs,$action,$tasktitleref) = @_;
    my $response;
    my $chgtotal = 0;
    my $chgok = 0;
    my $chgfail = 0;
    my $othdom = 0;
    my $locktotal = 0;
    my $lockok = 0;
    my $lockfail = 0;
    my $cid = $dom."_".$crs;
    my %chg = ();
    my %nochg = ();
    my %otherdom = ();
    my %lockchg = ();
    my %nolockchg = ();
    my $classlist = &Apache::loncoursedata::get_classlist($cid,$dom,$crs);
    my $endidx = &Apache::loncoursedata::CL_END;
    my $startidx = &Apache::loncoursedata::CL_START;
    my $ididx=&Apache::loncoursedata::CL_ID;
    my $secidx=&Apache::loncoursedata::CL_SECTION;
    my $typeidx=&Apache::loncoursedata::CL_TYPE;
    my $lockedidx=&Apache::loncoursedata::CL_LOCKEDTYPE;
    my @typechglist = ();
    my @lockchglist = ();
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['chgauto','chgmanual','lockchg','unlockchg']);
    if ($ENV{'form.chgauto'}) {
        if (ref($ENV{'form.chgauto'}) eq 'ARRAY') {
            push @typechglist, @{$ENV{'form.chgauto'}};
        } else {
            push @typechglist, $ENV{'form.chgauto'};
        }
    }
    if ($ENV{'form.chgmanual'}) {
        if (ref($ENV{'form.chgmanual'}) eq 'ARRAY') {
            push @typechglist, @{$ENV{'form.chgmanual'}};
        } else {
            push @typechglist, $ENV{'form.chgmanual'};
        }
    }
    if ($ENV{'form.lockchg'}) {
        if (ref($ENV{'form.lockchg'}) eq 'ARRAY') {
            push @lockchglist, @{$ENV{'form.lockchg'}};
        } else {
            push @lockchglist, $ENV{'form.lockchg'};
        }
    }
    if ($ENV{'form.unlockchg'}) {
        if (ref($ENV{'form.unlockchg'}) eq 'ARRAY') {
            push @lockchglist, @{$ENV{'form.unlockchg'}};
        } else {
            push @lockchglist, $ENV{'form.unlockchg'};
        }
    }
    foreach my $student (sort @typechglist) {
        my ($uname,$udom) = split/:/,$student;
        my $sdata    = $classlist->{$student};
        my $section  = $sdata->[$secidx];
        my $uid       = $sdata->[$ididx];
        my $start    = $sdata->[$startidx];
        my $end      = $sdata->[$endidx];
        my $type     = $sdata->[$typeidx];
        my $lock   = $sdata->[$lockedidx];
        my $newlock = $lock;
        $chgtotal ++;
        my $newtype = 'auto';
        my $change = 'auto';
        my $oldtype = 'manual';
        if ($type eq 'auto') { 
            $oldtype = 'auto';
            $newtype = '';
            $change = 'manual';
        }
        if ($udom eq $dom) {
            if ($newtype eq 'auto') {
                $newlock = '';
            } elsif ($newtype eq '') {
                $newlock = '1';
            }
            my $modreply = &Apache::lonnet::modify_student_enrollment($udom,$uname,$uid,'','','','',$section,$end,$start,$newtype,$newlock,$cid);
            if ($modreply eq 'ok') {
                $chgok ++;
                $chg{$student} = "Changed to $change";
            } else {
                $chgfail ++;
                $nochg{$student} = "Still set to $oldtype";
            } 
        } else {
            $othdom ++;
            $otherdom{$student} = "Still set to $oldtype"; 
        }
    }
    foreach my $student (@lockchglist) {
        my ($uname,$udom) = split/:/,$student;
        my $sdata    = $classlist->{$student};
        my $section  = $sdata->[$secidx];
        my $uid       = $sdata->[$ididx];
        my $start    = $sdata->[$startidx];
        my $end      = $sdata->[$endidx];
        my $type     = $sdata->[$typeidx];
        my $lock   = $sdata->[$lockedidx];
        my $newlock = 1;
        my $oldlockname = &mt('unlocked');
        my $newlockname = &mt('locked');
        $locktotal++;
        unless ($type eq 'auto') {
            if ($lock) {
                $newlock = '';
                $newlockname = &mt('unlocked');
                $oldlockname = &mt('locked'); 
            }
            my $lockreply = &Apache::lonnet::modify_student_enrollment($udom,$uname,$uid,'','','','',$section,$end,$start,$type,$newlock,$cid);
            if ($lockreply eq 'ok') {
                $lockok ++;
                $lockchg{$student} = 'Changed to '.$newlockname;
            } else {
                $lockfail ++;
                $nolockchg{$student} = 'Still set to '.$oldlockname;
            }
        }
    }
    if ($chgtotal > 0) {
        $response = "You requested a change in enrollment type for $chgtotal students.<br /><br />\n";
        $classlist = &Apache::loncoursedata::get_classlist($cid,$dom,$crs);
        if ($chgok > 0) {
            $response .= "The following $chgok changes were successful:<br />";
            $response .= &enrolltype_result(\%chg,$classlist,$endidx,$startidx,$ididx,$secidx,$typeidx);
        }
        if ($chgfail > 0) {
            $response .= "The following $chgfail students were not modified successfully:&nbsp;<br />";
            $response .= &enrolltype_result(\%nochg,$classlist,$endidx,$startidx,$ididx,$secidx,$typeidx);
        }
        if ($othdom > 0) {
            $response .= "The following $othdom students were not modified because students must be in the same LON-CAPA domain as the course, in order to be set to an enrollment type of 'auto':<br />"; 
            $response .= &enrolltype_result(\%otherdom,$classlist,$endidx,$startidx,$ididx,$secidx,$typeidx);
        }
        $response .= "<br /><br />";
    }
    if ($locktotal > 0) {
        $response .= "You requested locking/unlocking for $locktotal manually enrolled students.<br /><br />\n";
        $classlist = &Apache::loncoursedata::get_classlist($cid,$dom,$crs);
        if ($lockok > 0) {
            $response .= "The following $lockok changes were successful:<br />";
            $response .= &enrolltype_result(\%lockchg,$classlist,$endidx,$startidx,$ididx,$secidx,$typeidx,$lockedidx);
        }
        if ($lockfail > 0) {
            $response .= "The following $lockfail students were not modified successfully:&nbsp;<br />";
            $response .= &enrolltype_result(\%nolockchg,$classlist,$endidx,$startidx,$ididx,$secidx,$typeidx,$lockedidx);
        }
    }
    &print_reply($r,$response,$$tasktitleref{$action});
    return;
}

sub enrolltype_result {
    my ($changes,$classlist,$endidx,$startidx,$ididx,$secidx,$typeidx,$lockedidx) = @_;
    my $reply = "
            <table border='2'>
             <tr>
              <th>username</th>
              <th>domain</th>
              <th>ID</th>
              <th>student name</th>
              <th>section</th>
              <th>start date</th>
              <th>end date</th>
              <th>enrollment change</th>
             </tr>";
    foreach (sort keys %{$changes}) {
        my $sdata  = $classlist->{$_};
        my ($uname,$udom) = split/:/,$_;
        my $section  = $sdata->[$secidx];
        my $uid      = $sdata->[$ididx];
        my $start    = $sdata->[$startidx];
        my $end      = $sdata->[$endidx];
        my $type     = $sdata->[$typeidx];
        if (! defined($start) || $start == 0) {
            $start = &mt('none');
        } else {
            $start = &Apache::lonlocal::locallocaltime($start);
        }
        if (! defined($end) || $end == 0) {
            $end = &mt('none');
        } else {
            $end = &Apache::lonlocal::locallocaltime($end);
        }
        if (!defined($section) || ($section eq '')) {
            $section eq '&nbsp;';
        }
        if (!defined($uid) || ($uid eq '')) {
            $uid = '&nbsp';
        }
        $reply .= "
             <tr>
              <td>$uname</td>
              <td>$udom</td>
              <td>$uid</td>
              <td>".&Apache::loncommon::plainname($uname,$udom)."</td>
              <td>$section</td>
              <td>$start</td>
              <td>$end</td>
              <td>$$changes{$_}</td>
             </tr>";
    }
    $reply .= "</table>";
    return $reply;
}

sub warning_message {
    my ($dom,$crs,$caller) = @_;
    my %settings = &Apache::lonnet::get('environment',['internal.autoadds','internal.autodrops','internal.sectionnums','internal.crosslistings','internal.autostart','internal.autoend'],$dom,$crs);
    my $currend = '';
    my $currstart = '';
    my $currsecs = '';
    my $currxlists = '';
    my $warning = '';
    my $curradds = '';
    my $currdrops = '';
    if ( defined($settings{'internal.autoadds'}) ) {
        $curradds = $settings{'internal.autoadds'};
    }
    if (defined($settings{'internal.autodrops'}) ) {
        $currdrops = $settings{'internal.autodrops'};
    }
    if ( defined($settings{'internal.autostart'}) ) {
        $currstart = $settings{'internal.autostart'};
    }
    if ( defined($settings{'internal.autoend'}) ) {
        $currend = $settings{'internal.autoend'};
    }
    if ( defined($settings{'internal.sectionnums'}) ) {
        $currsecs = $settings{'internal.sectionnums'};
    }
    if ( defined($settings{'internal.crosslistings'}) ) {
        $currxlists = $settings{'internal.crosslistings'}
    }
    unless ($caller eq 'setdates') {
        if ( ($currstart eq '') && ($currend eq '') )  {
            $warning = "You <b>must</b> now use <a href='/adm/populate?action=setdates'>Change enrollment dates</a> to set a start date <i>and</i> an end date for the enrollment (or check the 'No end date' checkbox) for the nightly adds process to actually occur.</br></br>";
        }
    }
    unless ( ($caller eq 'sections') || ($caller eq 'crosslist') ) {
        if ( ($currsecs eq '') && ($currxlists eq '') ) {
            $warning .= "You <b>must</b> now use <a href='/adm/populate?action=sections'>Section settings</a> and/or <a href='/adm/populate?action=crosslist'>Change crosslistings</a> to choose at least one section of the course, or at least one crosslisted course which will contribute enrollment to this LON-CAPA course.  At present there are <b>NO</b> sections or crosslisted courses that are affiliated with this course that are set to contribute to the automated enrollment process.<br/><br/>";
        }
    }
    unless ( $caller eq 'chgsettings') {
        if ( (!$curradds) && (!$currdrops) ) {
            $warning .= "You <b>must</b> now use <a href='/adm/populate?action=chgsettings'>Automated adds/drops</a> to enable automated adds and/or drops if you want automatic enrollment updates to occur in this course.<br/><br/>.";
        }
    }
    return $warning;
}

sub print_reply () {
  my ($r,$response,$caller) = @_;
  $r->print("
            <br/><table width=\"100%\" border=\"0\" cellpadding=\"2\" cellspacing=\"2\">
             <tr>
              <td align=\"left\"><b>$caller</b> - result<br/><br/>$response</td>
             </tr>
            </table>
  ");
  return;
}

sub setup_date_selectors {
    my ($starttime,$endtime,$action) = @_;
    if (! defined($starttime)) {
        $starttime = time;
        if ($action eq 'setdates') {
            if (exists($ENV{'course.'.$ENV{'request.course.id'}.
                            '.default_enrollment_start_date'})) {
                $starttime = $ENV{'course.'.$ENV{'request.course.id'}.
                                  '.default_enrollment_start_date'};
            }
        }
    }
    if (! defined($endtime)) {
        $endtime = time+(6*30*24*60*60); # 6 months from now, approx
        if ($action eq 'setdates') {
            if (exists($ENV{'course.'.$ENV{'request.course.id'}.
                            '.default_enrollment_end_date'})) {
                $endtime = $ENV{'course.'.$ENV{'request.course.id'}.
                                '.default_enrollment_end_date'};
            }
        }
    }
    my $startdateform = &Apache::lonhtmlcommon::date_setter('enter',
                                                            'startdate',
                                                            $starttime);
    my $enddateform = &Apache::lonhtmlcommon::date_setter('enter',
                                                          'enddate',
                                                          $endtime);
    return ($startdateform,$enddateform);
}

sub get_dates_from_form {
    my $startdate;
    my $enddate;
    $startdate = &Apache::lonhtmlcommon::get_date_from_form('startdate');
    $enddate   = &Apache::lonhtmlcommon::get_date_from_form('enddate');
    if ( exists ($ENV{'form.no_end_date'}) ) {
        $enddate = 0;
    }
    return ($startdate,$enddate);
}

sub date_setting_table {
    my ($starttime,$endtime,$action) = @_;
    my ($startform,$endform) = &setup_date_selectors($starttime,$endtime,$action);
    my $perpetual = '<nobr><input type="checkbox" name="no_end_date"';
    if (($action eq 'setdates' && defined($endtime) && $endtime == 0) || (($action eq 'setaccess' || $action eq 'updatenow') && ($endtime eq '' || $endtime == 0)) ) {
        $perpetual .= ' checked';
    }
    $perpetual.= ' />'.' no ending date</nobr>';
    my $start_table = '';
    $start_table .= "<table>\n";
    $start_table .= '<tr><td align="right">Starting Date</td>'.
        '<td>'.$startform.'</td>'.
        '<td>&nbsp;</td>'."</tr>\n";
    $start_table .= "</table>";
    my $end_table = '';
    $end_table .= "<table>\n";
    $end_table .= '<tr><td align="right">Ending Date</td>'.
        '<td>'.$endform.'</td>'.
        '<td>'.$perpetual.'</td>'."</tr>\n";
    $end_table .= "</table>\n";
    return ($start_table, $end_table);
}

###################################################################
sub handler {
    my $r = shift;
    if ($r->header_only) {
        $r->content_type('text/html');
        $r->send_http_header;
        return OK;
    }
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['action','state']);
    #  Needs to be in a course
    if (! (($ENV{'request.course.fn'}) &&  (&Apache::lonnet::allowed('cst',$ENV{'request.course.id'})))) {
        # Not in a course, or not allowed to modify parms
        $ENV{'user.error.msg'}="/adm/populate:cst:0:0:Cannot drop or add students";
        return HTTP_NOT_ACCEPTABLE; 
    }
    # Start page
    $r->content_type('text/html');
    $r->send_http_header;

    my @tasks = ("information","chgsettings","setdates","setaccess","notify","crosslist","sections","photos","updatenow","viewclass");
                                                                                 
    my %tasklong = (
               information => "Task information",
               chgsettings => "Automated adds/drops",
               setdates => "Change enrollment dates",
               setaccess => "Change access dates",
               notify => "Notification of changes",
               crosslist => "Change crosslistings",
               sections => "Section settings",
               photos => "Student photos",
               updatenow => "Update roster now",
               newcross => "Add crosslistings",
               newsections => "Add sections",
               viewclass => "View students and change type",
    );
                                                                                 
    my %tasktitle = (
               chgsettings => "Changes to nightly automated enrollments",
               setdates => "Changes to first and/or last automated enrollment dates",
               setaccess => "Changes to default start and/or end dates for student access",
               notify => "Notification of enrollment changes",
               crosslist => "Changes to crosslistings",
               sections => "Changes to section settings",
               photos => "Automatic import of student photos",
               updatenow => "Immediate course roster update",
               newcross => "Adding new crosslisted courses",
               newsections => "Adding new course sections",
               viewclass => "Viewing class roster and enrollment type"
    );

    my $realm = '';
    if ( exists($ENV{'request.course.id'}) ) {
        $realm= $ENV{'course.'.$ENV{'request.course.id'}.'.description'};
    }
    unless ($realm) { $realm='&nbsp;'; }
    my $dom = $ENV{'course.'.$ENV{'request.course.id'}.'.domain'};
    my $crs = $ENV{'course.'.$ENV{'request.course.id'}.'.num'};
    
    #
    # Main switch on form.action and form.state, as appropriate
    #

    my $action = "information";
    if ( exists($ENV{'form.action'}) ) {
        $action = $ENV{'form.action'};
    }
    my $state = "choose";

    if ( exists($ENV{'form.state'}) ) {
        $state = $ENV{'form.state'};
    }

    if ($action eq "information") {
        $r->print(&header()); 
    } else {
        if ($state eq "choose") {
            $r->print(&choose_header($action));
        } else {
            if ($action eq "crosslist") {
                if ( exists($ENV{'form.numcross'}) ) {
                    if ( $ENV{'form.numcross'} > 0 ) {
                        $r->print(&choose_header($action));
                    } else {
                        $r->print(&header());
                    }
                } else {
                    $r->print(&header());
                }
            } elsif ($action eq "sections") {
                if ( exists($ENV{'form.numsec'}) ) {
                    if ( $ENV{'form.numsec'} > 0 ) {
                        $r->print(&choose_header($action));
                    } else {
                        $r->print(&header());
                    }
                } else {
                    $r->print(&header());
                }
            } else {
                $r->print(&header());
            }
        }
    }

    my $reply = 0;
    unless ($state eq "choose") { $reply = 1; }

    &print_mainbox($r,\%tasklong,$realm,$reply);
    &print_navmenu($r,\@tasks,\%tasklong);
    
    if (($state eq "choose") || ($action eq "information")) {
        &print_main_frame($r,$realm,$dom,$crs,\%tasktitle);
    } elsif ($action eq "chgsettings") {
        &print_chgsettings_response($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "setdates") {
        &print_setdates_response($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "setaccess") {
        &print_setaccess_response($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "notify") {
        &print_notify_response($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "sections") {
        &print_sections_menu($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "crosslist") {
        &print_crosslistings_menu($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "updatenow") {
        &print_update_result($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "photos") {
        &print_photos_response($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "newcross") {
        &print_crosslistings_response($r,$realm,$dom,$crs,$action,\%tasktitle);    
    } elsif ($action eq "newsections") {
        &print_sections_response($r,$realm,$dom,$crs,$action,\%tasktitle);
    } elsif ($action eq "viewclass") {
        &print_viewclass_response($r,$realm,$dom,$crs,$action,\%tasktitle);
    }
    &print_doc_base($r);  
    return OK;
}
###################################################################
1;
